; =====================================================================
; Sensormodul2 zur Temperatur- und Alarm-Erfassung mit ATmega8
; von Scott-Falk Hhn (s.huehn@soemtron.de)
; Version 1.00, letzte Bearbeitung am 19.07.2007
; =====================================================================
;
; Dieses Sensormodul kommuniziert mit bis zu 16 Temperatursensoren, die
; an bis zu 8 1-Wire-Busse angeschlossen werden knnen. Verwendbar sind
; folgende Typen: DS1820, DS18S20, DS18B20, DS1822, DS1920. Zustzlich
; sind noch 4 Alarmeingnge vorhanden. Ein LCD mit HD44780 Controller
; (4 Zeilen, 20 Zeichen) zeigt die Temperaturwerte aller Sensoren und
; weitere Daten an. Mit einem Taster knnen verschiedene Anzeige-Modi
; ausgewhlt und einige Parameter konfiguriert werden. Die ermittelten
; Daten werden folgendermaen ber RS232 mit 9600 Baud im ASCII Format
; gesendet:
;
; * Sensorbelegung:
;
; 33000200#     (Beispiel: 3 Sensoren an Bus1, 3 Sensoren an Bus2 und
;		2 Sensoren an Bus6)
;	  - Enter-Zeichen CR (13)
; --------  Anzahl der Sensoren an jedem 1-Wire Bus, von Bus1 bis Bus8;
;	    bei mehr als 9 Sensoren an einem Bus wird ein Buchstabe
;	    ausgegeben: aus 10 wird "A", aus 11 wird "B" usw.
;
; Diese Meldung wird nur einmal in der Startphase des Sensormoduls
; gesendet (ca. 10,5 Sekunden nach dem Einschalten).
;
; * Temperaturwerte:
;
; 2: 23.5#	(Temperatur 2 = 23.5C)
;        - Enter-Zeichen CR (13)
;   -----  Temperaturwert im Bereich -55.0 bis 125.0
;  -       Trennzeichen (Doppelpunkt)
; -        Sensornummer im Bereich 1-8 und a-h (Sensor 9-16)
;
; Das Minuszeichen kann an 3. oder 4. Stelle stehen. Die Daten werden
; fortlaufend in einem einstellbaren Intervall zwischen 100 und 1000ms
; gesendet. Ein kompletter Zyklus dauert dann bei 16 Sensoren je nach
; Intervall-Einstellung zwischen 2,6 und 17s.
;
; * Alarmmeldungen:
;
; C:1#		(Alarm 3 ein)
;    - Enter-Zeichen CR (13)
;   -  Alarmwert 0 =Alarm aus (Kontakt geffnet)
;		 1 =Alarm ein (Kontakt geschlossen)
;  -   Trennzeichen (Doppelpunkt)
; -    Alarmnummer, A entspricht Alarm 1, B entspreicht Alarm 2 usw.
;
; Eine Alarmmeldung wird bei jeder Statusnderung eines Alarmeingangs
; gesendet. Auerdem wird zu Beginn eines neuen Zyklus der aktuelle
; Status aller 4 Alarme gesendet.
;
; =====================================================================
;
; Einstellung der Konfigurationsbits (Fuse-Bits):
;
; CKOPT				 = 1
; CKSEL3, CKSEL2, CKSEL1, CKSEL0 = 1111  (Quarz 3-8 MHz)
; SUT1, SUT0			 = 11    (Quarz, langsame Betriebsspg.)
; BODLEVEL, BODEN		 = 00    (Brown-Out ein, 4,0 V)
;
; =====================================================================
;
; ----------------------
; Belegung der I/O-Ports
; ----------------------
;
; PortB0: Eingang Alarm 1 mit Pull-up, offen= Alarm aus, Low= Alarm ein
; PortB1: Eingang Alarm 2 mit Pull-up, offen= Alarm aus, Low= Alarm ein
; PortB2: Ausgang LCD, Anschluss D4
; PortB3: Ausgang LCD, Anschluss D5 (MOSI)
; PortB4: Ausgang LCD, Anschluss D6 (MISO)
; PortB5: Ausgang LCD, Anschluss D7 (SCK)
;
; PortC0: Ein/Ausgang Sensorbus 1
; PortC1: Ein/Ausgang Sensorbus 2
; PortC2: Ein/Ausgang Sensorbus 3
; PortC3: Ein/Ausgang Sensorbus 4
; PortC4: Ausgang LCD, Anschluss RS
; PortC5: Ausgang LCD, Anschluss E
;
; PortD0: Eingang Taster mit Pull-up
; PortD1: Ausgang RS-232, 9600 Baud
; PortD2: Eingang Alarm 3 mit Pull-up, offen= Alarm aus, Low= Alarm ein
; PortD3: Eingang Alarm 4 mit Pull-up, offen= Alarm aus, Low= Alarm ein
; PortD4: Ein/Ausgang Sensorbus 5
; PortD5: Ein/Ausgang Sensorbus 6
; PortD6: Ein/Ausgang Sensorbus 7
; PortD7: Ein/Ausgang Sensorbus 8
;
; ------------------
; Belegung der Timer
; ------------------
;
; Timer0: nicht genutzt
; Timer1: Interrupt alle 50ms
; Timer2: nicht genutzt
;
; -----------------------------------
; Konstanten und Registerdefinitionen
; -----------------------------------
;
.nolist
.include "m8def.inc"
.list
;
.equ	clock=	4000000	;Controller Taktfrequenz= 4,0MHz
.equ	tim1fr=	20	;Timer1 Frequenz= 20Hz (50ms)
.equ	baud=	9600	;RS-232 Baudrate= 9600
.equ	cr=	13	;ASCII-Code fr CR
.equ	keytio=	40	;Langer Tastendruck= 2s (50ms-Einheiten)
.equ	almtim=	60	;Alarmanzeige-Zeit= 3s (50ms-Einheiten)
.equ	setrtn=	1200	;Rckkehr aus Setup-Men= 60s (50ms-Einheiten)
.equ	maxsen=	16	;maximale Anzahl an Temperatursensoren (16)
;
.def	isreg=	r13	;Zwischenspeicher fr SREG bei Interruptbearb.
.def	itemp1=	r14	;Zwischenspeicher 1 fr Interruptroutine
.def	itemp2=	r24	;Zwischenspeicher 2 fr Interruptroutine
.def	flags=	r25	;Verschiedene Flags
			;Bit0: Timer-Flag, wird bei jedem Timer1-Inter-
			;      rupt gesetzt und vom Hauptprogramm
			;      wieder gelscht
			;Bit1: nderungs-Flag, zeigt an, dass die Tem-
			;      peraturanzeige aktualisiert werden muss
			;Bit2: Anzeige-Flag, sperrt die automatische
			;      Wechsel-Anzeige bei Benutzereingriff
			;Bit3: Tastenflag 1 - vorheriger Tasterstatus,
			;      wird vom Timer1-Interrupt verwaltet,
			;      enthlt Taster-Status vom vorherigen
			;      Timer1-Interrupt
			;Bit4: Tastenflag 2 - kurzer Tastendruck, wird
			;      vom Interrupt gesetzt und gelscht
			;Bit5: Tastenflag 3 - kurzer Tastendruck quit-
			;      tiert, wird vom Hauptprogramm gesetzt
			;      und vom Interrupt wieder gelscht
			;Bit6: Tastenflag 4 - langer Tastendruck, wird
			;      vom Interrupt gesetzt und gelscht
			;Bit7: Tastenflag 5 - langer Tastendruck quit-
			;      tiert, wird vom Hauptprogramm gesetzt
			;      und vom Interrupt wieder gelscht
;
; --------------
; SRAM Variablen
; --------------
;
.dseg
.org	0x60
;
rsbuff:	.byte	8	;RS-232 Puffer fr 8 Textzeichen (ASCII)
alstat:	.byte	4	;Alarmstatus fr 4 Alarme, wird vollstndig vom
			;Timer1-Interrupt gesteuert:
			;Bit0: aktuell gelesener Alarmstatus
			;Bit1: letzter stabiler Alarmstatus
			;Bit2: Bearbeitungs-Flag (zeigt an, dass Sta-
			;      tuswechsel schon bearbeitet wird)
			;Bit3: Melde-Flag, Info fr Hauptprogramm zum
			;      sofortigen Senden des Statuswechsels,
			;      wird dann vom Hauptprogramm gelscht
			;Bit4: Melde-Status, Info fr das Hauptprogramm
			;      ber den zu meldenden Status
acount:	.byte	4	;Alarmzeitzhler fr 4 Alarme, wird vom Inter-
			;rupt bei Statusnderung auf <altime> gesetzt,
			;auf 0 gezhlt, danach wird Melde-Flag gesetzt
sensid:	.byte	maxsen*8;8-Byte-ROM-IDs der 16 Temperatursensoren
senstb:	.byte	maxsen	;Sensor-Tabelle, enthlt fr jeden der 16 Sen-
			;soren die Nummer des 1-Wire-Busses (0-7), an
			;dem er angeschlossen ist, fr nicht vorhandene
			;Sensoren wird 0xff eingetragen
temval:	.byte	maxsen*5;16 Temperaturwerte mit je 5 ASCII-Zeichen
senflg:	.byte	maxsen	;Sensor-Flag fr 16 Temperatursensoren
			;Bit0: letzter Temperaturwert war 85,0 Grad
owbuff:	.byte	8	;1-Wire-Puffer fr empfangene 64-Bit-ROM-ID und
			;fr den Empfang des Scratchpad
adrtab:	.byte	8	;LCD-Adress-Tabelle fr Messwert-Positionen
senmap:	.byte	8	;ermittelte Sensorbelegung
sensrs:	.byte	1	;ermittelte Anzahl der Sensoren
sencnt:	.byte	1	;Sensorzhler, ist whrend der Mess-Phase 0 und
			;wird im Bereich von 1 bis <sensrs> gesetzt -
			;zeigt den aktuell bearbeiteten Sensor an
intcnt:	.byte	1	;Interruptzhler fr das Mess-Timing
modflg:	.byte	1	;Modus-Flag (wird im EEPROM gespeichert):
			;=0: Sensordaten werden aus EEPROM gelesen
			;>0: Sensordaten werden durch Suche ermittelt
intval:	.byte	1	;Sende-Intervall fr Temperaturdaten in 50ms-
			;Schritten (wird im EEPROM gespeichert), Werte-
			;Bereich von 2 bis 20 (100ms bis 1000ms)
altime:	.byte	1	;Alarm-Zeit in 50ms-Schritten (wird im EEPROM
			;gespeichert), Wertebereich von 2 bis 20
			;(100ms bis 1000ms)
lang:	.byte	1	;Men-Sprache (wird im EEPROM gespeichert):
			;=0: Englisch
			;>0: Deutsch
dimode:	.byte	1	;Anzeige-Modus:
			;0: Sensoren 1-8
			;1: Sensoren a-h (9-16)
			;2: Alarme
			;3: Status (Version, Sensoren, Belegung, Modus)
semode:	.byte	1	;Einstellungs-Modus (Setup):
			;0: aus
			;1: Modus: Direkt/Speicher
			;2: Intervall: 100, 250, 500, 750, 1000ms
			;3: Alarmzeit: 250, 500, 750, 1000ms
			;4: Sprache: Englisch/Deutsch
keycnt:	.byte	1	;Tasterzhler, wird bei Tastendruck auf
			;<keytio> gesetzt und auf 0 gezhlt, dient zur
			;Erkennung des langen Tastendrucks
intc16:	.byte	2	;16-Bit Interruptzhler, wird bei jedem Timer1-
			;Interrupt (alle 50ms) um 1 vermindert und bei
			;0 gestoppt, fr lange Warteschleifen und
			;automatische Rckkehr in den Automatik-Modus
dezval:	.byte	4	;Zwischenspeicher fr die Umrechnung von 16-Bit
			;Hex-Werten in 4 Dezimal-Stellen
flags2:	.byte	1	;Verschiedene Flags 2:
			;Bit0: Merker fr kurzen Tastendruck, wird fr
			;      Einstellungs-Modus bentigt, um das
			;      ffnen des Tasters zu erkennen
lastal:	.byte	1	;zuletzt ausgelster oder zurckgesetzter Alarm
			;im Bereich 0-3, wird fr die kurzzeitige
			;Alarm-Anzeige im Automatik-Modus bentigt
;
; =====================================================================
; Programmbereich
; =====================================================================
;
.cseg
.org	0
;
; Interrupt-Vektoren
;
reset:	rjmp	start		;Programm-Start
intr0:	reti			;External Interrupt Request 0
intr1:	reti			;External Interrupt Request 1
t2comp:	reti			;Timer/Counter2 Compare Match
t2ovf:	reti			;Timer/Counter2 Overflow
t1capt:	reti			;Timer/Counter1 Capture Event
t1coma:	rjmp	timer1		;Timer/Counter1 Compare Match A
t1comb:	reti			;Timer/Counter1 Compare Match B
t1ovf:	reti			;Timer/Counter1 Overflow
t0ovf:	reti			;Timer/Counter0 Overflow
spistc:	reti			;SPI, STC Serial Transfer Complete
usrxc:	reti			;RXC USART, Rx Complete
usudre:	reti			;UDRE USART Data Register Empty
ustxc:	reti			;TXC USART, Tx Complete
adc_cc:	reti			;ADC Conversion Complete
ee_rdy:	reti			;EEPROM Ready
anacom:	reti			;Analog Comparator
twi:	reti			;Two-wire Serial Interface
spmrdy:	reti			;Store Program Memory Ready
;
; Initialisierung SRAM, I/O-Ports, Hardware
;
start:	ldi	zl,0x60
	ldi	zh,0			;Anfangsadresse SRAM
	clr	r16
sramcl:	st	z+,r16			;Speicherzelle lschen
	cpi	zl,low(ramend+1)	;letzte Speicherzelle erreicht?
	brne	sramcl			;nein -> Schleife
	cpi	zh,high(ramend+1)	;letzte Speicherzelle erreicht?
	brne	sramcl			;nein -> Schleife
;
	ldi	r16,low(ramend)
	ldi	r17,high(ramend)
	out	spl,r16
	out	sph,r17			;Stackpointer setzen
;
	ldi	r16,0b00111100
	out	ddrb,r16		;PortB, B5-B2 Aus, B1-B0 Ein
	ldi	r16,0b00000011
	out	portb,r16		;PortB, B1-B0 Pullup ein
	ldi	r16,0b00110000
	out	ddrc,r16		;PortC, C5-C4 Aus, C3-C0 Ein
	clr	r16
	out	portc,r16		;PortC
	ldi	r16,0b00000010
	out	ddrd,r16		;PortD, nur TXD als Ausgang
	ldi	r16,0b00001111
	out	portd,r16		;PortD, Pullup fr Alarm/Taster
;
	ldi	r16,low((clock/(16*baud))-1)
	ldi	r17,high((clock/(16*baud))-1)
	out	ubrrh,r17		;USART Baudratenregister auf
	out	ubrrl,r16		;9600 Baud setzen
	ldi	r16,(1<<ursel)|(1<<ucsz1)|(1<<ucsz0)
	out	ucsrc,r16		;8 Datenbits, 1 Stoppbit
	ldi	r16,1<<txen
	out	ucsrb,r16		;Sender aktivieren
;
	ldi	r16,low((clock/(8*20))-1)
	ldi	r17,high((clock/(8*20))-1)
	out	ocr1ah,r17		;Timer1, Output Compare A
	out	ocr1al,r16		;fr 20Hz (50ms) setzen
	ldi	r16,(1<<wgm12)|(1<<cs11)
	out	tccr1b,r16		;Timer1, CTC-Modus, Vorteiler=8
	ldi	r16,1<<ocie1a
	out	timsk,r16		;OC1A-Interrupt aktivieren
;
	clr	flags			;alle Flags lschen
	rcall	w050ms			;50 ms warten
;
; Temperaturwerte initialisieren
;
	ldi	zl,low(temval)
	ldi	zh,high(temval)		;Zeiger auf Temperaturwerte
	ldi	r16,' '			;Leerzeichen
	ldi	r17,maxsen*5		;16 Werte mit je 5 Zeichen
cltval:	st	z+,r16			;Leerzeichen schreiben
	dec	r17			;alle Werte bearbeitet?
	brne	cltval			;nein -> Schleife
;
; Variablen aus EEPROM auslesen
;
	ldi	r16,low(emodfl)
	ldi	r17,high(emodfl)	;EEPROM-Adresse Modus-Flag
	rcall	eeread			;EEPROM lesen
	sts	modflg,r18		;Modus-Flag speichern
;
	ldi	r16,low(eintva)
	ldi	r17,high(eintva)	;EEPROM-Adresse Sende-Intervall
	rcall	eeread			;EEPROM lesen
	cpi	r18,21			;Sende-Intervall>20?
	brcs	reava1			;nein -> weiter
	ldi	r18,20			;sonst auf 20 setzen
reava1:	sts	intval,r18		;Sende-Intervall speichern
;
	ldi	r16,low(ealtim)
	ldi	r17,high(ealtim)	;EEPROM-Adresse Alarm-Zeit
	rcall	eeread			;EEPROM lesen
	cpi	r18,21			;Alarm-Zeit>20?
	brcs	reava2			;nein -> weiter
	ldi	r18,10			;sonst auf 10 setzen
reava2:	sts	altime,r18		;Alarm-Zeit speichern
;
	ldi	r16,low(elang)
	ldi	r17,high(elang)		;EEPROM-Adresse Men-Sprache
	rcall	eeread			;EEPROM lesen
	sts	lang,r18		;Men-Sprache speichern
;
; LCD Adress-Tabelle in RAM kopieren
;
	ldi	zl,low(2*lcdtab)	;Zeiger auf LCD-Adress-Tabelle
	ldi	zh,high(2*lcdtab)	;im Flash
	ldi	xl,low(adrtab)		;Zeiger auf LCD-Adress-Tabelle
	ldi	xh,high(adrtab)		;im RAM
	ldi	r17,8			;8 Werte kopieren
copadr:	lpm	r16,z+			;Wert aus Flash holen
	st	x+,r16			;in RAM schreiben
	dec	r17			;alle Zeichen kopiert?
	brne	copadr			;nein -> Schleife
;
; LCD initialisieren und Sonderzeichen programmieren
;
	clr	r17			;LCD in Befehlsmodus setzen
	ldi	r18,0x30		;Initialisierung
	rcall	lcd8bt			;im 8-Bit Mode ausgeben
	rcall	w004ms			;Warteschleife 4,1ms
	ldi	r18,0x30		;Initialisierung
	rcall	lcd8bt			;nochmals im 8-Bit Mode ausgeb.
	rcall	w100us			;Warteschleife 100s
	ldi	r18,0x30		;Initialisierung
	rcall	lcd8bt			;nochmals im 8-Bit Mode ausgeb.
	rcall	w040us			;Warteschleife 40s
	ldi	r18,0x20		;4-Bit Mode
	rcall	lcd8bt			;nochmals im 8-Bit Mode ausgeb.
	rcall	w040us			;Warteschleife 40s
	ldi	r18,0x28		;4-Bit Mode, 2-Line, 5x7 Font
	rcall	lcdw40			;ausgeben
	ldi	r18,0x0c		;Display ein, kein Cursor
	rcall	lcdw40			;ausgeben
	ldi	r18,0x06		;Increment, kein Scolling
	rcall	lcdw40			;ausgeben
;
	ldi	r18,0x40		;LCD-Adresse der Sonderzeichen
	clr	r17			;LCD in Befehlsmodus setzen
	rcall	lcdw40			;Adresse setzen
	ldi	r19,5*8			;40 Bytes fr 5 Sonderzeichen
	ldi	r17,1			;LCD in Datenmodus setzen
	ldi	zl,low(2*xchars)	;Zeiger auf Bitmaps fr
	ldi	zh,high(2*xchars)	;Sonderzeichen (Pfeile)
lcdini:	lpm	r18,z+			;Datenbyte aus Flash holen
	rcall	lcdw40			;an LCD ausgeben
	dec	r19			;alle Zeilen ausgegeben?
	brne	lcdini			;nein -> Schleife
;
; Temperatursensor-Tabelle lschen
;
initab:	ldi	xl,low(senstb)
	ldi	xh,high(senstb)		;Zeiger auf Sensortabelle
	ldi	r16,0xff		;Startwert 0xff
init10:	st	x+,r16			;in Puffer schreiben
	cpi	xl,low(senstb+maxsen)	;Tabellenende erreicht?
	brne	init10			;nein -> Schleife
;
	lds	r16,modflg		;Modus-Flag holen
	tst	r16			;Sensordaten aus EEPROM laden?
	brne	search			;nein -> Sensor-Suche starten
;
; Sensordaten aus EEPROM auslesen
;
	ldi	r20,maxsen		;maximale Sensorenanzahl (16)
	ldi	r16,low(esenid)
	ldi	r17,high(esenid)	;EEPROM-Adresse der Sensor-IDs
	ldi	xl,low(sensid)
	ldi	xh,high(sensid)		;Zeiger auf Sensor-IDs im RAM
loadi1:	ldi	r19,8			;jeweils 8 ID-Bytes
loadi2:	rcall	eeread			;EEPROM lesen
	st	x+,r18			;gelesenes Byte speichern
	inc	r16			;nchste EEPROM-Adresse L
	brne	loadi3			;bertrag? nein -> weiter
	inc	r17			;nchste EEPROM-Adresse H
loadi3:	dec	r19			;alle 8 Bytes gelesen?
	brne	loadi2			;nein -> Schleife
	dec	r20			;alle Sensoren gelesen?
	brne	loadi1			;nein -> Schleife
;
	ldi	r20,maxsen		;maximale Sensorenanzahl (16)
	ldi	r16,low(esentb)
	ldi	r17,high(esentb)	;EEPROM-Adresse der Sensor-Tab.
	ldi	xl,low(senstb)
	ldi	xh,high(senstb)		;Zeiger auf Sensor-Tab. im RAM
loadt1:	rcall	eeread			;EEPROM lesen
	st	x+,r18			;gelesenes Byte speichern
	inc	r16			;nchste EEPROM-Adresse L
	brne	loadt2			;bertrag? nein -> weiter
	inc	r17			;nchste EEPROM-Adresse H
loadt2:	dec	r20			;alle Sensoren gelesen?
	brne	loadt1			;nein -> Schleife
	rjmp	inmain			;sonst Ende
;
; 1-Wire Temperatursensoren suchen und ROM-IDs in SRAM schreiben,
; Algorithmus nach Application Note 187 von Dallas/Maxim
;
search:	ldi	r22,0			;Sensornummer=0 setzen
	ldi	r20,0			;Busnummer=0 setzen
	ldi	zl,low(sensid)
	ldi	zh,high(sensid)		;Zeiger auf Sensor-IDs
seanew:	clr	r21			;LastDiscrepancy lschen
	clr	r9			;LastDeviceFlag lschen
	clr	r8			;ReturnValue lschen
searom:	rcall	owres			;Reset senden
	brcs	sea700			;Presence? nein -> Suchende
	tst	r9			;LastDeviceFlag gesetzt?
	brne	sea700			;ja -> Suchende
	clr	r11
	inc	r11			;Bit-Number=1 setzen
	clr	r10			;Last-Zero=0 setzen
	ldi	r19,0xf0		;Search-ROM-Kommando (0xf0)
	rcall	owwrit			;senden
sea100:	rcall	owbrea			;einzelnes Bit lesen (Bit A)
	rol	r18			;Bit A sichern
	andi	r18,0x01		;restliche Bits lschen
	rcall	owbrea			;einzelnes Bit lesen (Bit B)
	rol	r19			;Bit B sichern
	andi	r19,0x01		;restliche Bits lschen
	mov	r17,r19			;Bit B kopieren
	and	r17,r18			;Beide Bits=1?
	brne	sea700			;ja -> weiter
	mov	r17,r19			;Bit B kopieren
	or	r17,r18			;Beide Bits=0?
	breq	sea400			;ja -> weiter
	mov	r16,r18			;Bit A speichern
sea200:	bst	r16,0			;Bit sichern
	rcall	bitsto			;in Puffer einschieben
	bld	r16,0			;gesichertes Bit holen
	ror	r16			;und in Carry schieben
	rcall	owbwri			;Bit in Carry senden
	inc	r11			;Bit-Number erhhen
	mov	r16,r11			;Bit-Index kopieren
	cpi	r16,65			;Endwert erreicht (>64)?
	brcs	sea100			;nein -> Schleife
	mov	r21,r10			;LastDiscrepancy=Last-Zero
	tst	r21			;LastDiscrepancy=0?
	brne	sea300			;nein -> weiter
	inc	r9			;sonst LastDeviceFlag setzen
sea300:	rcall	crc			;CRC der ROM-ID prfen
	tst	r17			;CRC-Wert=0 (ok)?
	brne	sea700			;nein -> nchster Sensor
	inc	r8			;Return-Value setzen
	rjmp	sea800			;Ende
;
sea400:	cp	r11,r21			;Bit-Number=Last-Discrepancy?
	brne	sea500			;nein -> weiter
	ldi	r16,1			;sonst "1"-Bit setzen
sea450:	cpi	r16,0			;Search-Direction=0?
	brne	sea200			;nein -> Bit speichern/senden
	mov	r10,r11			;Last-Zero=Bit-Number
	rjmp	sea200			;Bit speichern/senden
;
sea500:	brcs	sea600			;Bit-Number<Last-Discrepancy?->
	clr	r16			;sonst "0"-Bit setzen
	rjmp	sea450			;Bit speichern und senden
;
sea600:	lds	r16,owbuff+7		;entprechendes Bit von letzter
	andi	r16,0x01		;Suche holen und maskieren
	rjmp	sea450			;Bit speichern und senden
;
sea700:	clr	r21			;LastDiscrepancy=0 setzen
	clr	r9			;LastDeviceFlag=0 setzen
	clr	r8			;ReturnValue=0 setzen
;
sea800:	tst	r8			;wurde ein Sensor gefunden?
	breq	sea900			;nein -> nchsten Bus testen
;
	lds	r16,owbuff+7		;sonst Family-Code holen
	cpi	r16,0			;gltiger Family-Code (>0)?
	breq	sea900			;nein -> nchster Bus
	cpi	r16,0x10		;Family-Code=10h (DS18S20)?
	breq	sea810			;ja -> bearbeiten
	cpi	r16,0x28		;Family-Code=28h (DS18B20)?
	breq	sea810			;ja -> bearbeiten
	cpi	r16,0x22		;Family-Code=22h (DS1822)?
	brne	sea830			;nein -> nchster Sensor
;
sea810:	ldi	xl,low(owbuff)
	ldi	xh,high(owbuff)		;Zeiger auf 1-Wire-Puffer
	ldi	r19,8			;8 Bytes kopieren
sea820:	ld	r18,x+			;Bytes aus 1-Wire-Puffer holen
	st	z+,r18			;und ROM-ID-Feld speichern
	dec	r19			;alle Bytes bearbeitet?
	brne	sea820			;nein -> Schleife
	ldi	xl,low(senstb)
	ldi	xh,high(senstb)		;Zeiger auf Sensortabelle
	add	xl,r22			;Sensornummer addieren
	adc	xh,r19			;Tabellenplatz berechnen
	st	x,r20			;Busnummer speichern
	inc	r22			;Sensornummer erhhen
	cpi	r22,maxsen		;Maximum 16 erreicht?
	breq	inmain			;ja -> Ende
;
sea830:	tst	r9			;weitere Sensoren am Bus?
	brne	sea900			;nein -> nchsten Bus testen
	rjmp	searom			;sonst weitere Sensoren lesen
;
sea900:	inc	r20			;Busnummer erhhen
	cpi	r20,8			;Maximum 8 erreicht?
	breq	inmain			;ja -> Ende
	rjmp	seanew			;neue Suche beginnen
;
; Initialisierung der Haupprogrammschleife
;
inmain:	rcall	getsen			;Anzahl der Sensoren ermitteln
	rcall	getmap			;Sensorbelegung ermitteln
	rcall	dstatu			;Status auf LCD ausgeben
;
	ldi	r16,210
	clr	r17			;Interruptzhler auf
	sts	intc16,r16		;210*50ms (10,5s) setzen
	sts	intc16+1,r17		;(warten auf Anzeigemodul 1)
	sei				;Interrupts freigeben
inm010:	lds	r16,intc16		;Interrupt-Zhler L holen
	lds	r17,intc16+1		;Interrupt-Zhler H holen
	or	r16,r17			;Zhler abgelaufen?
	brne	inm010			;nein -> warten
;
	ldi	r16,cr			;CR laden und zur Synchronisie-
	rcall	rssend			;rung mit dem Empfnger senden
	ldi	xl,low(senmap)
	ldi	xh,high(senmap)		;Zeiger auf Sensorbelegung
	ldi	r17,8			;8 Zeichen senden
inm020:	ld	r16,x+			;Zeichen holen
	rcall	rssend			;Zeichen senden
	dec	r17			;alle Zeichen gesendet?
	brne	inm020			;nein -> Schleife
	ldi	r16,cr			;CR-Code laden
	rcall	rssend			;und senden
;
	cli				;Interrupts sperren
	ldi	r16,21
	clr	r17
	sts	intc16,r16		;Interruptzhler auf
	sts	intc16+1,r17		;20*50ms (1s) setzen
	sei				;Interrupts wieder freigeben
inm030:	lds	r16,intc16		;Interrupt-Zhler L holen
	lds	r17,intc16+1		;Interrupt-Zhler H holen
	or	r16,r17			;Zhler abgelaufen?
	brne	inm030			;nein -> warten
;
	ldi	zl,low(alstat)
	ldi	zh,high(alstat)		;Zeiger auf Alarmstatus
	ldi	r20,4			;4 Alarme bearbeiten
	cli				;Interrupts sperren
inm040:	ld	r19,z			;Alarmstatus holen
	cbr	r19,0b00001000		;Melde-Flag lschen
	st	z+,r19			;Alarmstatus wieder speichern
	dec	r20			;alle Alarme bearbeitet?
	brne	inm040			;nein -> Schleife
;
	cbr	flags,0b00000001	;Interrupt-Flag lschen
	sei				;Interrupts wieder freigeben
;
;
; =====================================================================
; Hauptprogrammschleife
; =====================================================================
;
; Aktuelle Alarmmeldungen ausgeben
;
main:	ldi	zl,low(alstat)
	ldi	zh,high(alstat)		;Zeiger auf Alarmstatus
	ldi	r20,0			;aktuelle Alarmnummer
ma0010:	ld	r19,z			;Alarmstatus holen
	bst	r19,3			;Melde-Flag gesetzt?
	brtc	ma0030			;nein -> weiter
;
	bst	flags,2			;Automatik-Anzeige aktiv?
	brts	ma0020			;nein -> kein Anzeigewechsel
;
	sts	lastal,r20		;Alarmnummer speichern als
	ldi	r16,low(almtim)		;letzten Alarm
	ldi	r17,high(almtim)
	sts	intc16,r16		;Interrupt-Zhler auf
	sts	intc16+1,r17		;Alarm-Anzeige-Zeit setzen
;
ma0020:	cli				;Interrupts sperren
	ld	r19,z			;Alarmstatus nochmals holen
	cbr	r19,0b00001000		;Melde-Flag lschen
	st	z,r19			;Alarmstatus wieder speichern
	sei				;Interrupts wieder aktivieren
	swap	r19			;Alarmstatus an Bitposition 0
	andi	r19,0x01		;restliche Bits entfernen
	rcall	rsaout			;Alarmmeldung senden
	sbr	flags,0b00000010	;nderungs-Flag setzen
;
ma0030:	adiw	zl,1			;Zeiger auf nchsten Alarm
	inc	r20			;Alarmnummer erhhen
	cpi	r20,4			;alle Alarme bearbeitet?
	brne	ma0010			;nein -> Schleife
;
; Interrupt-Zhler prfen fr automatische Rckkehr aus Einstellungs-
; Men in die Automatik-Anzeige
;
	lds	r16,semode		;Einstellungs-Modus holen
	tst	r16			;Einstellung-Modus aktiv?
	breq	ma0050			;nein -> berspringen
	lds	r16,intc16		;Interrupt-Zhler L holen
	lds	r17,intc16+1		;Interrupt-Zhler H holen
	or	r16,r17			;Zhler=0?
	brne	ma0050			;nein -> bersrpingen
	sts	semode,r16		;Einstellungs-Modus=0 setzen
	sts	dimode,r16		;Anzeige-Modus=0 setzen
	cbr	flags,0b00000100	;Anzeige-Automatik aktivieren
	sbr	flags,0b00000010	;nderungs-Flag setzen
;
; Zyklus-Steuerung nach folgendem Muster:
;
; Zeitpunkt       0ms: Messung starten
; Zeitpunkt     900ms: Messung beenden
; Zeitpunkt    1000ms: Sensornummer 1 setzen / Sensor 1 lesen
; Zeitpunkt +<intval>: Sensornummer 2 setzen / Sensor 2 lesen
; Zeitpunkt +<intval>: Sensornummer n setzen / Sensor n lesen
; Zeitpunkt +<intval>: Sensornummer 0 setzen und Sprung zu Zeitpunkt 0
;
ma0050:	bst	flags,0			;Interrupt-Flag gesetzt?
	brtc	ma0130			;nein -> berspringen
;
	cbr	flags,0b00000001	;Interrupt-Flag wieder lschen
ma0100:	lds	r17,intcnt		;Interruptzhler holen
	inc	r17			;erhhen
	sts	intcnt,r17		;und wieder speichern
	lds	r16,sencnt		;Sensorzhler holen
	tst	r16			;Sensorzhler=0?
	brne	ma0120			;nein -> weiter testen
	cpi	r17,1			;Mess-Start-Zeitpunkt?
	breq	ma0200			;ja -> bearbeiten
	cpi	r17,19			;Mess-Stop-Zeitpunkt?
	breq	ma0400			;ja -> bearbeiten
	cpi	r17,21			;Sensor-Abfrage beginnen?
	brne	ma0130			;nein -> Ende
;
ma0110:	clr	r17			;Interruptzhler lschen
	sts	intcnt,r17		;und wieder speichern
	inc	r16			;Sensorzhler erhhen
	sts	sencnt,r16		;und wieder speichern
	lds	r18,sensrs		;Sensoranzahl holen
	inc	r18			;erhhen
	cp	r16,r18			;letzter Sensor erreicht?
	brcs	ma1000			;nein -> Sensor lesen
	clr	r16			;sonst Sensorzhler lschen
	sts	sencnt,r16		;und wieder speichern
	sbr	flags,0b00000010	;nderungs-Flag setzen
	rjmp	ma0100			;neuer Zyklus
;
ma0120:	lds	r18,intval		;Sende-Intervall holen
	cp	r17,r18			;Endwert erreicht?
	brcc	ma0110			;ja -> bearbeiten
ma0130:	rjmp	ma2000			;sonst Ende
;
; Neue Messung starten
;
ma0200:	clr	r20			;Busnummer=0 setzen
ma0210:	ldi	xl,low(senstb)
	ldi	xh,high(senstb)		;Zeiger auf Sensor-Tabelle
ma0220:	ld	r19,x+			;verwendeten Bus des Sensors
	cp	r19,r20			;holen, =aktueller Bus?
	breq	ma0230			;ja -> Messung auf Bus starten
	cpi	xl,low(senstb+maxsen)	;Tabellenende erreicht?
	brne	ma0220			;nein -> nchster Tabellenplatz
	rjmp	ma0240			;sonst nchster Bus
;
ma0230:	cli				;Interrupts sperren
	rcall	owres			;1-Wire-Reset senden
	brcs	ma0240			;Presence? nein -> Fehler
	ldi	r19,0xcc		;ROM-Kommando "Skip ROM" senden
	rcall	owwrit			;(alle Sensoren ansprechen)
	ldi	r19,0x44		;Funktion-Kommando "Convert T"
	rcall	owwrit			;senden und fr sichere Messung
	rcall	owstpu			;Strong-Pullup aktivieren
ma0240:	sei				;Interrupts wieder freigeben
	inc	r20			;nchsten Bus bearbeiten
	cpi	r20,8			;letzter Bus erreicht?
	brne	ma0210			;nein -> Schleife
;
; Alarmmeldungen erneut senden
;
	ldi	zl,low(alstat)
	ldi	zh,high(alstat)		;Zeiger auf Alarmstatus
	ldi	r20,0			;aktuelle Alarmnummer
ma0300:	ld	r19,z+			;Alarmstatus holen
	swap	r19			;Alarmstatus an Bitposition 0
	andi	r19,0x01		;restliche Bits entfernen
	rcall	rsaout			;Alarmmeldung senden
	inc	r20			;Alarmnummer erhhen
	cpi	r20,4			;alle Alarme bearbeitet?
	brne	ma0300			;nein -> Schleife
	rjmp	ma2000
;
; Messung stoppen
;
ma0400:	clr	r20			;Busnummer=0 setzen
ma0410:	ldi	xl,low(senstb)
	ldi	xh,high(senstb)		;Zeiger auf Sensor-Tabelle
ma0420:	ld	r19,x+			;verwendeten Bus des Sensors
	cp	r19,r20			;holen, =aktueller Bus?
	breq	ma0430			;ja -> Messung auf Bus stoppen
	cpi	xl,low(senstb+maxsen)	;Tabellenende erreicht?
	brne	ma0420			;nein -> nchster Tabellenplatz
	rjmp	ma0440			;sonst nchster Bus
;
ma0430:	cli				;Interrupts sperren
	rcall	owres			;1-Wire-Reset,Strong-Pullup aus
	sei				;Interrupts wieder freigeben
ma0440:	inc	r20			;nchsten Bus bearbeiten
	cpi	r20,8			;letzter Bus erreicht?
	brne	ma0410			;nein -> Schleife
	rjmp	ma2000
;
; Sensor-Daten holen, Scratchpad aus Sensor auslesen
;
ma1000:	dec	r16			;Sensornummer korrigieren
;
ma1100:	mov	r17,r16			;Sensor-Nummer kopieren
	mov	r22,r16			;Sensor-Nummer sichern
	cpi	r17,8			;Sensor-Nummer>8?
	brcc	ma1110			;ja -> als Buchstabe ausgeben
	ldi	r17,49
	add	r17,r16			;in ASCII (1-8) wandeln
	rjmp	ma1120			;ausgeben
ma1110:	ldi	r17,89
	add	r17,r16			;in ASCII (a-h) wandeln
ma1120:	sts	rsbuff,r17		;in LCD-Puffer schreiben
	ldi	r17,':'			;Doppelpunkt
	sts	rsbuff+1,r17		;in LCD-Puffer schreiben
	ldi	r17,' '			;Leerzeichen
	sts	rsbuff+2,r17		;in LCD-Puffer schreiben
;
	ldi	xl,low(senstb)
	ldi	xh,high(senstb)		;Zeiger auf Sensor-Tabelle
	clr	r17			;H-Byte=0 fr Tabellenplatz
	add	xl,r16			;Tabellenplatz berechnen
	adc	xh,r17			;bertrag addieren
	ld	r20,x			;zugehrige Busnummer holen
	cpi	r20,0xff		;Sensor vorhanden?
	brne	ma1130			;ja -> weiter
	rjmp	ma1500			;sonst Fehler
;
ma1130:	ldi	xl,low(sensid)
	ldi	xh,high(sensid)		;Zeiger auf Sensor-ROM-IDs
	add	r16,r16
	add	r16,r16
	add	r16,r16			;Sensor-Nummer*8 als Offset
	add	xl,r16			;Tabellenplatz berechnen
	adc	xh,r17			;bertrag addieren
	adiw	xl,7			;Zeiger auf Family-Code
	ld	r7,x+			;Code holen, Zeiger auf Ende
;
	cli				;Interrupts sperren
	rcall	owres			;1-Wire-Reset senden
	sei				;Interrupts wieder freigeben
	brcc	ma1140			;Presence? ja -> weiter
	rjmp	ma1500			;sonst Fehler
;
ma1140:	cli				;Interrupts wieder sperren
	ldi	r19,0x55		;ROM-Kommando "Match ROM"
	rcall	owwrit			;senden
	ldi	r21,8			;8 Bytes ROM-ID senden
ma1150:	ld	r19,-x			;Byte aus Tabelle holen
	rcall	owwrit			;Byte senden
	dec	r21			;alle Bytes gesendet?
	brne	ma1150			;nein -> Schleife
	ldi	r19,0xbe		;Funktion "Read Scratchpad"
	rcall	owwrit			;senden
	ldi	xl,low(owbuff+8)
	ldi	xh,high(owbuff+8)	;Zeiger auf 1-Wire-Pufferende
	ldi	r21,8			;8 Bytes Scratchpad einlesen
ma1160:	rcall	owread			;Byte lesen
	st	-x,r19			;Byte in Puffer speichern
	dec	r21			;alle Bytes gelesen?
	brne	ma1160			;nein -> Schleife
	rcall	owread			;CRC-Byte lesen
	sei				;Interrupts wieder freigeben
	mov	r21,r19			;gelesenen CRC-Wert sichern
	rcall	crc			;CRC vom Scratchpad ermitteln
	cp	r17,r21			;war CRC-Wert ok?
	brne	ma1170			;nein -> Fehler
	lds	r16,owbuff		;zustzlich Count_Per_C holen
	cpi	r16,0x10		;und prfen,Count_Per_C=10h?
	breq	ma1180			;ja -> weiter
ma1170:	rjmp	ma1500			;sonst Fehler
;
ma1180:	ldi	r16,0x10		;Family-Code 0x10 (DS18S20)
	cp	r7,r16			;Family-Code ok?
	brne	ma1250			;nein -> and. Family bearbeiten
;
; DS1820/DS18S20/DS1920: Gelesene Daten aufbereiten, Erhhung der Mess-
; Genauigkeit durch Algorithmus nach Datenblatt; die Formel wurde etwas
; modifiziert, so dass mit ganzzahligen Werten gerechnet werden kann
; und keine Divisionsroutine ntig ist:
; Temp = (160 * Temp_Read - 40 + 10*(Count_Per_C - Count_Remain)) / 16
; Das Ergebnis liegt dann in Zehntel Grad vor.
;
ma1200:	lds	r17,owbuff+6		;Temperaturwert H holen
	lds	r16,owbuff+7		;Temperaturwert L holen
	ror	r17
	ror	r16			;LSB abschneiden (Temp_Read)
	ldi	r17,160			;Wert fr hhere Genauigkeit
	mulsu	r16,r17			;mit 160 multiplizieren
	movw	xl,r0			;Ergebnis kopieren
	sbiw	xl,40			;40 subtrahieren
	ldi	r16,16			;Count_Per_C laden (immer 16)
	lds	r17,owbuff+1		;Count_Remain holen
	sub	r16,r17			;Count_Per_C - Count_Remain
	add	r16,r16			;Wert *2
	mov	r17,r16			;kopieren
	add	r16,r16			;Wert nochmals *2 (*4)
	add	r16,r16			;Wert nochmals *2 (*8)
	add	r16,r17			;*2-Wert addieren (*10)
	clr	r17			;H-Byte=0
	add	xl,r16			;zur vorherigen Rechnung add.
	adc	xh,r17			;bertrag addieren
	asr	xh
	ror	xl			;Wert /2
	asr	xh
	ror	xl			;Wert /2 (/4)
	asr	xh
	ror	xl			;Wert /2 (/8)
	asr	xh
	ror	xl			;Wert /2 (/16)
	brcc	ma1210			;runden? nein -> weiter
	adiw	xl,1			;sonst Ergebnis aufrunden
ma1210:	mov	r20,xh			;Vorzeichen sichern
	bst	xh,7			;Ergebnis negativ?
	brtc	ma1300			;nein -> berspringen
	clr	r16			;Wert 0x10000 zur Wandlung ins
	clr	r17			;Zweierkomplement laden
	sub	r16,xl			;L subtrahieren
	sbc	r17,xh			;H mit bertrag subtrahieren
	movw	xl,r16			;ins Ergebnis-Register kopieren
	rjmp	ma1300			;in ASCII wandeln
;
; DS18B20/DS1822: Gelesene Daten aufbereiten, Umrechnung des 12-Bit
; Binrwertes in einen Dezimalwert in Zehntel Grad nach folgender
; Formel: Temp = Temp_Register * 5 / 8
;
ma1250:	lds	r16,owbuff+3		;Configuration Register holen
	cpi	r16,0x7f		;Sensor auf 12-Bit eingestellt?
	breq	ma1260			;ja -> weiter
	rjmp	ma1500			;sonst Fehler
ma1260:	lds	xh,owbuff+6		;Temperaturwert H holen
	lds	xl,owbuff+7		;Temperaturwert L holen
	mov	r20,xh			;Vorzeichen sichern
	bst	xh,7			;Temperaturwert negativ?
	brtc	ma1270			;nein -> weiter
	clr	r16			;Wert 0x10000 zur Wandlung ins
	clr	r17			;Zweierkomplement laden
	sub	r16,xl			;L subtrahieren
	sbc	r17,xh			;H mit bertrag subtrahieren
	movw	xl,r16			;ins Ergebnis-Register kopieren
ma1270:	movw	r16,xl			;Temperaturwert kopieren
	add	xl,xl
	adc	xh,xh			;Temperaturwert verdoppeln
	add	xl,xl
	adc	xh,xh			;nochmals verdoppeln (*4)
	add	xl,r16
	adc	xh,r17			;und Originalwert addieren (*5)
	asr	xh
	ror	xl			;Wert /2
	asr	xh
	ror	xl			;Wert /2 (/4)
	asr	xh
	ror	xl			;Wert /2 (/8)
	brcc	ma1300			;runden? nein -> weiter
	adiw	xl,1			;sonst Ergebnis aufrunden
;
; Ergebnis in Dezimal-ASCII umwandeln
;
ma1300:	clr	r16			;0-Wert fr 16-Bit Subtraktion
	clr	r18			;ASCII Stelle 2 lschen
	clr	r19			;ASCII Stelle 1 lschen
ma1310:	mov	r17,xl			;Binrzahl zwischenspeichern L
	subi	xl,100			;100 subtrahieren
	sbc	xh,r16			;bertrag subtrahieren
	brcs	ma1320			;war Zahl>99? nein -> weiter
	inc	r18			;ASCII Stelle 2 erhhen
	cpi	r18,10			;bertrag?
	brcs	ma1310			;nein -> Schleife
	clr	r18			;sonst ASCII Stelle 2 lschen
	inc	r19			;und ASCII Stelle 1 erhhen
	rjmp	ma1310			;Schleife
;
ma1320:	mov	xl,r17			;letzten Binrwert L holen
	clr	r17			;ASCII Stelle 3 lschen
ma1330:	cpi	xl,10			;Wert>9?
	brcs	ma1340			;nein -> Rest auswerten
	inc	r17			;sonst ASCII Stelle 3 erhhen
	subi	xl,10			;Wert um 10 vermindern
	rjmp	ma1330			;Schleife
ma1340:	mov	r16,xl			;Rest in ASCII Stelle 4 legen
	ori	r16,0x30		;Ergebnis in ASCII wandeln
	ori	r17,0x30
	ori	r18,0x30
	ori	r19,0x30
;
; Ergebnis aufbereiten
;
	cpi	r19,'0'			;ASCII Stelle 1 =0?
	brne	ma1350			;nein -> weiter
	ldi	r19,' '			;sonst durch Leerzeichen ers.
	cpi	r18,'0'			;hchste Ziffer=0?
	brne	ma1350			;nein -> berspringen
	ldi	r18,' '			;sonst durch Leerzeichen ers.
ma1350:	bst	r20,7			;ist Temperaturwert negativ?
	brtc	ma1370			;nein -> berspringen
	cpi	r18,' '			;Stelle 2 = Leerzeichen?
	breq	ma1360			;ja -> durch Minus ersetzen
	ldi	r19,'-'			;sonst Stelle 1 = Minuszeichen
	rjmp	ma1370
ma1360:	ldi	r18,'-'			;Stelle 2 = Minuszeichen
;
; Ergebnis in RS-232 Puffer und Wertetabelle legen
;
ma1370:	sts	rsbuff+2,r19		;ASCII Stelle 1 in RS232-Puffer
	sts	rsbuff+3,r18		;ASCII Stelle 2 in RS232-Puffer
	sts	rsbuff+4,r17		;ASCII Stelle 3 in RS232-Puffer
	ldi	r20,'.'			;Dezimalpunkt
	sts	rsbuff+5,r20		;in RS232-Puffer schreiben
	sts	rsbuff+6,r16		;ASCII Stelle 4 in RS232-Puffer
;
	mov	r21,r22			;gesicherte Sensornummer holen
	add	r21,r21
	add	r21,r21			;Adress-Offset von Wertetabelle
	add	r21,r22			;berechnen (Sensornummer * 5)
	clr	r23			;H-Byte=0
	ldi	zl,low(temval)
	ldi	zh,high(temval)		;Zeiger auf Wertetabelle
	add	zl,r21			;Tabellenplatz
	adc	zh,r23			;berechnen
	st	z+,r19			;ASCII-Stelle 1 in Tabelle
	st	z+,r18			;ASCII-Stelle 2 in Tabelle
	st	z+,r17			;ASCII-Stelle 3 in Tabelle
	ldi	r20,','			;Komma
	st	z+,r20			;in Tabelle schreiben
	st	z+,r16			;ASCII-Stelle 4 in Tabelle
	sbr	flags,0b00000010	;nderungs-Flag setzen
;
; Flags bearbeiten, Wert auf 85,0 Grad prfen
;
	ldi	xl,low(senflg)
	ldi	xh,high(senflg)		;Zeiger auf Sensorflags
	add	xl,r22			;gesicherte Sensornummer
	adc	xh,r23			;addieren, bertrag
	ld	r17,x			;Sensorflags holen
;
	lds	r16,rsbuff+2		;ASCII Stelle 1 holen
	cpi	r16,' '			;Wert=0?
	brne	ma1390			;nein -> Flag lschen
	lds	r16,rsbuff+3		;ASCII Stelle 2 holen
	cpi	r16,'8'			;Wert=8?
	brne	ma1390			;nein -> Flag lschen
	lds	r16,rsbuff+4		;ASCII Stelle 3 holen
	cpi	r16,'5'			;Wert=5?
	brne	ma1390			;nein -> Flag lschen
	lds	r16,rsbuff+6		;ASCII Stelle 4 holen
	cpi	r16,'0'			;Wert=0?
	brne	ma1390			;nein -> Flag lschen
;
	bst	r17,0			;85-Grad-Flag gesetzt?
	brts	ma1400			;ja -> Daten ausgeben
	sbr	r17,0b00000001		;sonst 85-Grad-Flag setzen
	st	x,r17			;Sensorflags wieder speichern
	rjmp	ma2000			;Datenausgabe berspringen
;
ma1390:	cbr	r17,0b00000001		;85-Grad-Flag lschen
	st	x,r17			;Sensorflags wieder speichern
;
; Daten ber RS-232 senden
;
ma1400:	ldi	xl,low(rsbuff)
	ldi	xh,high(rsbuff)		;Zeiger auf RS232-Puffer
	ldi	r17,7			;7 Zeichen senden
ma1410:	ld	r16,x+			;Zeichen holen
	rcall	rssend			;Zeichen senden
	dec	r17			;alle Zeichen gesendet?
	brne	ma1410			;nein -> Schleife
	ldi	r16,cr			;CR-Code laden
	rcall	rssend			;und senden
	rjmp	ma2000
;
; Fehler - Wert in Tabelle lschen
;
ma1500:	mov	r21,r22			;gesicherte Sensornummer holen
	add	r21,r21
	add	r21,r21			;Adress-Offset von Wertetabelle
	add	r21,r22			;berechnen (Sensornummer * 5)
	clr	r23			;H-Byte=0
	ldi	zl,low(temval)
	ldi	zh,high(temval)		;Zeiger auf Wertetabelle
	add	zl,r21			;Tabellenplatz
	adc	zh,r23			;berechnen
	ldi	r20,' '			;Leerzeichen
	st	z+,r20			;ASCII-Stelle 1 in Tabelle
	st	z+,r20			;ASCII-Stelle 2 in Tabelle
	st	z+,r20			;ASCII-Stelle 3 in Tabelle
	st	z+,r20			;Trennzeichen in Tabelle
	st	z+,r20			;ASCII-Stelle 4 in Tabelle
	sbr	flags,0b00000010	;nderungs-Flag setzen
;
; Taster: je nach Mode (Normal oder Einstellungen) verzweigen
;
ma2000:	cli				;Interrupts sperren
	lds	r16,semode		;Einstellungs-Modus holen
	tst	r16			;Einstellungs-Modus aktiv?
	brne	ma2200			;ja -> bearbeiten
;
; Taster: Normal-Modus, kurzen Tastendruck abfragen und bearbeiten
;
	bst	flags,4			;kurzer Tastendruck?
	brtc	ma2100			;nein -> Ende
	bst	flags,5			;kurzer Tastendruck quittiert?
	brts	ma2100			;ja -> Ende
;
	sbr	flags,0b00100010	;Quittieren + nderungs-Flag
	bst	flags,2			;Anzeige-Automatik gesperrt?
	brts	ma2010			;ja -> weiter
	sbr	flags,0b00000100	;Anzeige-Automatik sperren
	clr	r16			;Anzeige-Modus=0 setzen
	sts	dimode,r16		;Anzeige-Modus speichern
	rjmp	ma2100
;
ma2010:	lds	r16,dimode		;Anzeige-Modus holen
	inc	r16			;erhhen
	sts	dimode,r16		;und wieder speichern
	cpi	r16,1			;Anzeige-Modus=1?
	brne	ma2020			;nein -> weiter
	lds	r17,sensrs		;sonst Anzahl Sensoren holen
	cpi	r17,9			;mehr als 8 Sensoren?
	brcc	ma2020			;ja -> weiter
	inc	r16			;sonst Anz-Modus 1 berspringen
	sts	dimode,r16		;und speichern
;
ma2020:	cpi	r16,4			;Bereich berschritten?
	brcs	ma2100			;nein -> Ende
	clr	r16			;sonst Anzeige-Modus=0 setzen
	sts	dimode,r16		;und speichern
	cbr	flags,0b00000100	;Anzeige-Automatik freigeben
;
; Taster: Normal-Modus, langen Tastendruck abfragen und bearbeiten
;
ma2100:	bst	flags,6			;langer Tastendruck?
	brtc	ma2110			;nein -> Ende
	bst	flags,7			;langer Tastendruck quittiert?
	brts	ma2110			;ja -> Ende
;
	sbr	flags,0b10000000	;langen Tastendruck quittieren
	ldi	r16,1			;Einstellungs-Modus 1 setzen
	sts	semode,r16		;und speichern
	rcall	setinc			;Interrupt-Zhler setzen
ma2110:	sei				;Interrupts wieder freigeben
	rjmp	ma3000
;
; Taster: Einstellungs-Modus, langen Tastendruck abfragen u. bearbeiten
;
ma2200:	bst	flags,6			;langer Tastendruck?
	brtc	ma2210			;nein -> Ende
	bst	flags,7			;langer Tastendruck quittiert?
	brtc	ma2220			;nein -> bearbeiten
;
ma2210:	rjmp	ma2500
;
ma2220:	sbr	flags,0b10000000	;langen Tastendruck quittieren
	lds	r16,flags2		;Flags 2 holen
	cbr	r16,0b00000001		;Merker lschen
	sts	flags2,r16		;Flags 2 wieder speichern
	rcall	setinc			;Interrupt-Zhler setzen
	sei				;Interrupts wieder freigeben
;
	lds	r16,semode		;Einstellungs-Modus holen
	cpi	r16,1			;Modus-Einstellung?
	brne	ma2300			;nein -> weiter testen
;
	lds	r18,modflg		;sonst Modus-Flag holen
	tst	r18			;Direkt-Modus aktiv?
	brne	ma2230			;ja -> weiter
	ser	r18			;sonst in Direkt-Modus wechseln
	rjmp	ma2240			;Modus speichern
;
ma2230:	rcall	savsen			;Sensordaten im EEPROM speich.
	clr	r18			;in Speicher-Modus wechseln
ma2240:	sts	modflg,r18		;Modus-Flag wieder speichern
	ldi	r16,low(emodfl)
	ldi	r17,high(emodfl)	;EERPOM-Adresse Modus-Flag
	rcall	eewrit			;Modus-Flag speichern
	rjmp	ma3000
;
ma2300:	cpi	r16,2			;Sende-Intervall-Einstellung?
	brne	ma2400			;nein -> weiter testen
;
	lds	r18,intval		;Sende-Intervall holen
	cpi	r18,5			;Wert>=5?
	brcc	ma2310			;ja -> weiter
	ldi	r18,5			;sonst Wert=5 setzen
	rjmp	ma2350			;speichern
ma2310:	cpi	r18,10			;Wert>=10?
	brcc	ma2320			;ja -> weiter
	ldi	r18,10			;sonst Wert=10 setzen
	rjmp	ma2350			;speichern
ma2320:	cpi	r18,15			;Wert>=15?
	brcc	ma2330			;ja -> weiter
	ldi	r18,15			;sonst Wert=15 setzen
	rjmp	ma2350			;speichern
ma2330:	cpi	r18,20			;Wert>=20?
	brcc	ma2340			;ja -> weiter
	ldi	r18,20			;sonst Wert=20 setzen
	rjmp	ma2350			;speichern
ma2340:	ldi	r18,2			;Wert=2 setzen
ma2350:	sts	intval,r18		;neues Sende-Intervall speich.
	ldi	r16,low(eintva)
	ldi	r17,high(eintva)	;EERPOM-Adresse Sende-Intervall
	rcall	eewrit			;Sende-Intervall speichern
	rjmp	ma3000
;
ma2400:	cpi	r16,3			;Alarm-Zeit-Einstellung?
	brne	ma2450			;nein -> weiter
;
	lds	r18,altime		;Alarm-Zeit holen
	cpi	r18,10			;Wert>10?
	brcc	ma2410			;ja -> weiter
	ldi	r18,10			;sonst Wert=10 setzen
	rjmp	ma2440			;speichern
ma2410:	cpi	r18,15			;Wert>15?
	brcc	ma2420			;ja -> weiter
	ldi	r18,15			;sonst Wert=15 setzen
	rjmp	ma2440			;speichern
ma2420:	cpi	r18,20			;Wert>20?
	brcc	ma2430			;ja -> weiter
	ldi	r18,20			;sonst Wert=20 setzen
	rjmp	ma2440			;speichern
ma2430:	ldi	r18,5			;Wert=5 setzen
ma2440:	sts	altime,r18		;neue Alarm-Zeit speichern
	ldi	r16,low(ealtim)
	ldi	r17,high(ealtim)	;EERPOM-Adresse Alarm-Zeit
	rcall	eewrit			;Alarm-Zeit speichern
	rjmp	ma3000
;
ma2450:	lds	r18,lang		;Sprach-Einstellung holen
	tst	r18			;Sprache Deutsch?
	brne	ma2460			;ja -> weiter
	ser	r18			;sonst zu Deutsch wechseln
	rjmp	ma2470			;Sprache speichern
;
ma2460:	clr	r18			;zu Englisch wechseln
ma2470:	sts	lang,r18		;Sprache wieder speichern
	ldi	r16,low(elang)
	ldi	r17,high(elang)		;EERPOM-Adresse Sprache
	rcall	eewrit			;Sprache speichern
	rjmp	ma3000
;
; Taster: Einstellungs-Modus, kurzen Tastendruck abfragen und bearbei-
; ten; Aktion wird erst nach Loslassen des Tasters ausgelst
;
ma2500:	bst	flags,4			;kurzer Tastendruck?
	brtc	ma2510			;nein -> weiter
	bst	flags,5			;kurzer Tastendruck quittiert?
	brts	ma2520			;ja -> Ende
;
	sbr	flags,0b00100000	;Tastendruck quittieren
	lds	r16,flags2		;Flags 2 holen
	sbr	r16,0b00000001		;Merker setzen
	sts	flags2,r16		;Flags 2 wieder speichern
	rcall	setinc			;Interrupt-Zhler setzen
	rjmp	ma2520			;Ende
;
ma2510:	lds	r16,flags2		;Flags 2 holen
	bst	r16,0			;Merker gesetzt?
	brts	ma2530			;ja -> weiter
ma2520:	sei				;Interrupts wieder freigeben
	rjmp	ma3000			;Ende
;
ma2530:	cbr	r16,0b00000001		;Merker lschen
	sts	flags2,r16		;Flags 2 wieder speichern
	sei				;Interrupts wieder freigeben
	lds	r16,semode		;Einstellungs-Modus holen
	inc	r16			;Modus erhhen
	sts	semode,r16		;und wieder speichern
	cpi	r16,5			;Bereich berschritten?
	brcs	ma3000			;nein -> Ende
	clr	r16			;Sonst Modus=0 setzen
	sts	semode,r16		;und speichern
	sts	dimode,r16		;Anzeige-Modus=0 setzen
	cli				;Interrupts sperren
	sts	intc16,r16
	sts	intc16+1,r16		;Interrupt-Zhler lschen
	sei				;Interrupts wieder freigeben
	cbr	flags,0b00000100	;Anzeige-Automatik aktivieren
	sbr	flags,0b11110010	;Tasten quittieren, nd.-Flag
;
; Automatische Steuerung der Temperaturanzeige
;
ma3000:	lds	r16,semode		;Einstellungsmodus holen
	tst	r16			;Einstellungsmodus aktiv?
	brne	ma3200			;ja -> bearbeiten
;
	bst	flags,1			;nderungs-Flag gesetzt?
	brts	ma3010			;ja -> weiter
	rjmp	ma4000			;sonst Ende
;
ma3010:	cbr	flags,0b00000010	;nderungs-Flag wieder lschen
	bst	flags,2			;Anzeige-Automatik gesperrt?
	brts	ma3100			;ja -> manuelle Anzeige
;
	lds	r16,intc16		;Interrupt-Zhler L holen
	lds	r17,intc16+1		;Interrupt-Zhler H holen
	or	r16,r17			;Interrupt-Zhler=0?
	breq	ma3020			;ja -> weiter
	rcall	dalarm			;sonst auf Alarmanzeige umsch.
	rjmp	ma4000
;
ma3020:	lds	r22,sencnt		;aktuelle Sensornummer holen
	tst	r22			;Mess-Phase aktiv (Sensor=0)?
	brne	ma3030			;nein -> weiter
	lds	r22,sensrs		;sonst Sensoranzahl holen
	tst	r22			;sind Sensoren vorhanden?
	brne	ma3030			;ja -> weiter
	ser	r22			;sonst ungltigen Wert setzen
	rjmp	ma3040			;Sensoren 1-8 ausgeben
;
ma3030:	dec	r22			;korrigieren
	cpi	r22,8			;Sensornummer>7?
	brcc	ma3050			;ja -> berspringen
ma3040:	rcall	dsens1			;sonst Sensoren 1-8 ausgeben
	rjmp	ma4000
;
ma3050:	rcall	dsens2			;Sensoren 9-16 ausgeben
	rjmp	ma4000
;
; Manuelle Anzeige
;
ma3100:	lds	r16,dimode		;Anzeige-Modus holen
	tst	r16			;Anzeige-Modus=0?
	brne	ma3110			;nein -> weiter testen
	rcall	dsens1			;sonst Sensoren 1-8 anzeigen
	rjmp	ma4000
;
ma3110:	cpi	r16,1			;Anzeige-Modus=1?
	brne	ma3120			;nein -> weiter testen
	rcall	dsens2			;sonst Sensoren 9-16 anzeigen
	rjmp	ma4000
;
ma3120:	cpi	r16,2			;Anzeige-Modus=2?
	brne	ma3130			;nein -> weiter testen
	rcall	dalarm			;sonst Alarme anzeigen
	rjmp	ma4000
;
ma3130:	cpi	r16,3			;Anzeige-Modus=3?
	brne	ma3140			;nein -> Ende
	rcall	dstatu			;sonst Status anzeigen
ma3140:	rjmp	ma4000
;
; Einstellungs-Anzeige: Modus
;
ma3200:	cpi	r16,1			;Modus-Einstellung?
	brne	ma3300			;nein -> weiter testen
;
	lds	r18,adrtab		;LCD-Adresse Zeile 1
	clr	r17			;LCD in Befehlsmodus setzen
	rcall	lcdw40			;Adresse setzen
	ldi	r20,20			;Zeichenzhler setzen
	ldi	r19,11			;Text 11 (Modus:)
	rcall	lcdtxt			;ausgeben
	rcall	lcdlin			;Zeilenende lschen
;
	lds	r18,adrtab+1		;LCD-Adresse Zeile 2
	clr	r17			;LCD in Befehlsmodus setzen
	rcall	lcdw40			;Adresse setzen
	ldi	r20,20			;Zeichenzhler setzen
	ldi	r19,6			;Text 6 (Pfeil rechts)
	rcall	lcdtxt			;ausgeben
;
	ldi	r19,4			;Text 4 (Direkt)
	lds	r16,modflg		;Modus-Flag holen
	tst	r16			;Modus Direkt?
	brne	ma3210			;nein -> weiter
	ldi	r19,5			;sonst Text 5 (Speicher)
ma3210:	rcall	lcdtxt			;ausgeben
	ldi	r19,7			;Text 7 (Pfeil links)
	rcall	lcdtxt			;ausgeben
	rcall	lcdlin			;Zeilenende lschen
	rjmp	ma3700			;Menzeilen ausgeben
;
; Einstellungs-Anzeige: Sende-Intervall
;
ma3300:	cpi	r16,2			;Intervall-Eistellung?
	brne	ma3400			;nein -> weiter testen
;
	lds	r18,adrtab		;LCD-Adresse Zeile 1
	clr	r17			;LCD in Befehlsmodus setzen
	rcall	lcdw40			;Adresse setzen
	ldi	r20,20			;Zeichenzhler setzen
	ldi	r19,12			;Text 12 (Sende-Intervall:)
	rcall	lcdtxt			;ausgeben
	rcall	lcdlin			;Zeilenende lschen
;
	lds	r18,adrtab+1		;LCD-Adresse Zeile 2
	clr	r17			;LCD in Befehlsmodus setzen
	rcall	lcdw40			;Adresse setzen
	ldi	r20,20			;Zeichenzhler setzen
	ldi	r19,6			;Text 6 (Pfeil rechts)
	rcall	lcdtxt			;ausgeben
;
	lds	r16,intval		;Sende-Intervall holen
	ldi	r18,50			;Multiplikator
	mul	r16,r18			;Ergebnis in Millisekunden
	rcall	lcddez			;Ergebnis auf LCD ausgeben
	ldi	r19,16			;Text 16 (ms)
	rcall	lcdtxt			;ausgeben
	ldi	r19,7			;Text 7 (Pfeil links)
	rcall	lcdtxt			;ausgeben
	rcall	lcdlin			;Zeilenende lschen
	rjmp	ma3700			;Menzeilen ausgeben
;
; Einstellungs-Anzeige: Alarm-Zeit
;
ma3400:	cpi	r16,3			;Alarmzeit-Eistellung?
	brne	ma3500			;nein -> weiter testen
;
	lds	r18,adrtab		;LCD-Adresse Zeile 1
	clr	r17			;LCD in Befehlsmodus setzen
	rcall	lcdw40			;Adresse setzen
	ldi	r20,20			;Zeichenzhler setzen
	ldi	r19,13			;Text 13 (Alarm-Zeit:)
	rcall	lcdtxt			;ausgeben
	rcall	lcdlin			;Zeilenende lschen
;
	lds	r18,adrtab+1		;LCD-Adresse Zeile 2
	clr	r17			;LCD in Befehlsmodus setzen
	rcall	lcdw40			;Adresse setzen
	ldi	r20,20			;Zeichenzhler setzen
	ldi	r19,6			;Text 6 (Pfeil rechts)
	rcall	lcdtxt			;ausgeben
;
	lds	r16,altime		;Alarm-Zeit holen
	ldi	r18,50			;Multiplikator
	mul	r16,r18			;Ergebnis in Millisekunden
	rcall	lcddez			;Ergebnis auf LCD ausgeben
	ldi	r19,16			;Text 16 (ms)
	rcall	lcdtxt			;ausgeben
	ldi	r19,7			;Text 7 (Pfeil links)
	rcall	lcdtxt			;ausgeben
	rcall	lcdlin			;Zeilenende lschen
	rjmp	ma3700			;Menzeilen ausgeben
;
; Einstellungs-Anzeige: Sprache
;
ma3500:	cpi	r16,4			;Sprach-Einstellung?
	brne	ma4000			;nein -> Ende
;
	lds	r18,adrtab		;LCD-Adresse Zeile 1
	clr	r17			;LCD in Befehlsmodus setzen
	rcall	lcdw40			;Adresse setzen
	ldi	r20,20			;Zeichenzhler setzen
	ldi	r19,14			;Text 14 (Sprache:)
	rcall	lcdtxt			;ausgeben
	rcall	lcdlin			;Zeilenende lschen
;
	lds	r18,adrtab+1		;LCD-Adresse Zeile 2
	clr	r17			;LCD in Befehlsmodus setzen
	rcall	lcdw40			;Adresse setzen
	ldi	r20,20			;Zeichenzhler setzen
	ldi	r19,6			;Text 6 (Pfeil rechts)
	rcall	lcdtxt			;ausgeben
;
	ldi	r19,15			;Text 15 (Deutsch)
	rcall	lcdtxt			;ausgeben
	ldi	r19,7			;Text 7 (Pfeil links)
	rcall	lcdtxt			;ausgeben
	rcall	lcdlin			;Zeilenende lschen
;
; Einstellungs-Anzeige: Menzeilen
;
ma3700:	lds	r18,adrtab+2		;LCD-Adresse Zeile 3
	clr	r17			;LCD in Befehlsmodus setzen
	rcall	lcdw40			;Adresse setzen
	ldi	r20,20			;Zeichenzhler setzen
	ldi	r19,8			;Text 8 (Lang -> ndern)
	rcall	lcdtxt			;ausgeben
	rcall	lcdlin			;Zeilenende lschen
;
	lds	r18,adrtab+3		;LCD-Adresse Zeile 4
	clr	r17			;LCD in Befehlsmodus setzen
	rcall	lcdw40			;Adresse setzen
	ldi	r20,20			;Zeichenzhler setzen
	ldi	r19,9			;Text 9 (Kurz -> Weiter)
	lds	r16,semode		;Einstellungs-Modus holen
	cpi	r16,4			;Sprach-Einstellung aktiv?
	brne	ma3710			;nein -> weiter
	ldi	r19,10			;sonst Text 10 (Kurz -> Ende)
ma3710:	rcall	lcdtxt			;ausgeben
	rcall	lcdlin			;Zeilenende lschen
;
; Sonstiges
;
ma4000:	rjmp	main
;
; =====================================================================
; Unterprogramm-Routinen
; =====================================================================
;
; ----------------
; Anzeige-Routinen
; ----------------
;
; Sensoranzeige: Sensoren 1-8
; Register: r16,r17,r18,r19,r20,r21,x,z
;	    r22 <- aktuell bearbeiteter Sensor (fr Markierung)
;
dsens1:	ldi	xl,low(temval)
	ldi	xh,high(temval)		;Zeiger auf Wertetabelle
	ldi	zl,low(sentxt*2)
	ldi	zh,high(sentxt*2)	;Zeiger auf Sensortexte
	clr	r19			;Sensorzhler=0 setzen
	clr	r20			;Wertezhler=0 setzen
	rjmp	dse010
;
; Sensoranzeige: Sensoren 9-16
; Register: r16,r17,r18,r19,r20,r21,x,z
;	    r22 <- aktuell bearbeiteter Sensor (fr Markierung)
;
dsens2:	ldi	xl,low(temval+8*5)
	ldi	xh,high(temval+8*5)	;Zeiger auf Wertetabelle
	ldi	zl,low(sentxt*2+8*4)
	ldi	zh,high(sentxt*2+8*4)	;Zeiger auf Sensortexte
	ldi	r19,8			;Sensorzhler=8 setzen
	clr	r20			;Wertezhler=0 setzen
;
dse010:	movw	r2,xl			;Zeiger sichern
	ldi	xl,low(adrtab)
	ldi	xh,high(adrtab)		;Zeiger auf LCD-Adresstabelle
	clr	r21			;H-Byte=0
	add	xl,r20
	adc	xh,r21			;Tabellenplatz ermitteln
	ld	r18,x			;LCD-Adresse holen
	movw	xl,r2			;Zeiger wiederherstellen
	clr	r17			;LCD in Befehlsmode setzen
	rcall	lcdw40			;ausgeben
;
	ldi	r17,1			;LCD in Datenmodus setzen
	ldi	r21,4			;4 Zeichen ausgeben
dse020:	lpm	r18,z+			;Zeichen holen
	rcall	lcdw40			;ausgeben
	dec	r21			;alle Zeichen bearbeitet?
	brne	dse020			;nein -> Schleife
;
	ldi	r21,5			;5 Zeichen ausgeben
dse030:	ld	r18,x+			;Zeichen holen
	rcall	lcdw40			;ausgeben
	dec	r21			;alle Zeichen bearbeitet?
	brne	dse030			;nein -> Schleife
;
	ldi	r18,' '			;Leerzeichen laden
	bst	flags,2			;Anzeige-Automatik gesperrt?
	brts	dse040			;ja -> Leerzeichen ausgeben
	cp	r19,r22			;sonst: aktueller Sensor?
	brne	dse040			;nein -> berspringen
	ldi	r18,0			;sonst Pfeil links laden
dse040:	rcall	lcdw40			;ausgeben
;
	inc	r19			;Sensorzhler erhhen
	inc	r20			;Wertezhler erhhen
	cpi	r20,8			;alle Daten ausgegeben?
	brne	dse010			;nein -> Schleife
	rjmp	dsymbl			;Manuell-Symbol ausgeben
;
; Alarmanzeige
; Register: r16,r17,r18,r19,r20,r21,r22,r23,x,z
;
dalarm:	ldi	xl,low(alstat)
	ldi	xh,high(alstat)		;Zeiger auf Alarm-Status
	ldi	zl,low(almtxt*2)
	ldi	zh,high(almtxt*2)	;Zeiger auf Alarmtexte
	clr	r21			;Alarmzhler=0 setzen
;
	ldi	r23,0xff		;letzten Alarm nicht zeigen
	bst	flags,2			;Automatik-Anzeige gesperrt?
	brts	dal010			;ja -> weiter
	lds	r16,intc16		;Interrupt-Zhler L holen
	lds	r17,intc16+1		;Interrupt-Zhler H holen
	or	r16,r17			;Zhler=0?
	breq	dal010			;ja -> weiter
	lds	r23,lastal		;sonst letzten Alarm holen
;
dal010:	movw	r2,xl			;Zeiger sichern
	ldi	xl,low(adrtab)
	ldi	xh,high(adrtab)		;Zeiger auf LCD-Adresstabelle
	clr	r22			;H-Byte=0
	add	xl,r21
	adc	xh,r22			;Tabellenplatz ermitteln
	ld	r18,x			;LCD-Adresse holen
	movw	xl,r2			;Zeiger wiederherstellen
	clr	r17			;LCD in Befehlsmode setzen
	rcall	lcdw40			;ausgeben
;
	ldi	r20,20			;Zeichenzhler setzen
	ldi	r17,1			;LCD in Datenmodus setzen
	ldi	r19,12			;12 Zeichen ausgeben
dal020:	lpm	r18,z+			;Zeichen holen
	rcall	lcdw40			;ausgeben
	dec	r20			;Zeichenzhler-1
	dec	r19			;alle Zeichen bearbeitet?
	brne	dal020			;nein -> Schleife
	ldi	r18,' '			;Leerzeichen laden
	rcall	lcdw40			;ausgeben
	dec	r20			;Zeichenzhler-1
;
	ldi	r18,2			;Sonderzeichen "offen"
	ld	r16,x+			;Alarm-Flags holen
	bst	r16,4			;Alarm aktiv?
	brtc	dal030			;nein -> weiter
	ldi	r18,3			;Sonderzeichen "geschlossen"
dal030:	rcall	lcdw40			;ausgeben
	dec	r20			;Zeichenzhler-1
;
	movw	r2,zl			;Zeiger sichern
	cp	r21,r23			;Alarm krzlich ausgelst?
	brne	dal040			;nein -> weiter
	ldi	r19,7			;sonst Text 7 (Pfeil links)
	rcall	lcdtxt			;Text ausgeben
dal040:	movw	zl,r2			;Zeiger wiederherstellen
	rcall	lcdlin			;Zeilenende lschen
;
	inc	r21			;nchster Alarm
	cpi	r21,4			;alle Alarme bearbeitet?
	brne	dal010			;nein -> Schleife
	rjmp	dsymbl			;Manuell-Symbol ausgeben
;
; Statusanzeige: Versionsnummer, Anzahl der Sensoren, Sensorbelegung,
; Modus (Direkt oder Speicher)
; Register: r16,r17,r18,r19,r20,x,z
;
dstatu:	lds	r18,adrtab		;LCD-Adresse Zeile 1
	clr	r17			;LCD in Befehlsmodus setzen
	rcall	lcdw40			;Adresse setzen
	ldi	r20,20			;Zeichenzhler setzen
	ldi	r19,0			;Text 0 (Versionsnummer)
	rcall	lcdtxt			;ausgeben
	rcall	lcdlin			;Zeilenende lschen
;
	lds	r18,adrtab+1		;LCD-Adresse Zeile 2
	clr	r17			;LCD in Befehlsmodus setzen
	rcall	lcdw40			;Adresse setzen
	ldi	r20,20			;Zeichenzhler setzen
	ldi	r19,1			;Text 1 (Sensoren:)
	rcall	lcdtxt			;ausgeben
;
	lds	r18,sensrs		;Anzahl der Sensoren holen
	cpi	r18,10			;mehr als 10 Sensoren?
	brcs	dst010			;nein -> weiter
	ldi	r18,'1'			;sonst ASCII "1" laden
	rcall	lcdw40			;ausgeben
	dec	r20			;Zeichenzhler-1
	lds	r18,sensrs		;Anzahl der Sensoren holen
	subi	r18,10			;10 subtrahieren
dst010:	ori	r18,0x30		;in ASCII wandeln
	rcall	lcdw40			;ausgeben
	dec	r20			;Zeichenzhler-1
	rcall	lcdlin			;Zeilenende lschen
;
	lds	r18,adrtab+2		;LCD-Adresse Zeile 3
	clr	r17			;LCD in Befehlsmodus setzen
	rcall	lcdw40			;Adresse setzen
	ldi	r20,20			;Zeichenzhler setzen
	ldi	r19,2			;Text 2 (Belegung:)
	rcall	lcdtxt			;ausgeben
;
	ldi	zl,low(senmap)
	ldi	zh,high(senmap)		;Zeiger auf Sensorbelegung
	ldi	r19,8			;8 Zeichen ausgeben
	ldi	r17,1			;LCD in Datenmodus setzen
dst020:	ld	r18,z+			;Zeichen holen
	rcall	lcdw40			;ausgeben
	dec	r20			;Zeichenzhler-1
	dec	r19			;alle Zeichen ausgegeben?
	brne	dst020			;nein -> Schleife
	rcall	lcdlin			;Zeilenende lschen
;
	lds	r18,adrtab+3		;LCD-Adresse Zeile 4
	clr	r17			;LCD in Befehlsmodus setzen
	rcall	lcdw40			;Adresse setzen
	ldi	r20,20			;Zeichenzhler setzen
	ldi	r19,3			;Text 3 (Modus:)
	rcall	lcdtxt			;ausgeben
	ldi	r19,4			;Text 4 (Direkt)
	lds	r16,modflg		;Modus-Flag holen
	tst	r16			;Daten aus EEPROM verwenden?
	brne	dst030			;nein -> weiter
	ldi	r19,5			;sonst Text 5 (Speicher)
dst030:	rcall	lcdtxt			;ausgeben
	rcall	lcdlin			;Zeilenende lschen
;
; Alle manuellen Anzeigen: Ausgabe eines Symbols unten rechts
;
dsymbl:	bst	flags,2			;Automatik-Anzeige gesperrt?
	brtc	dsy010			;nein -> Ende
;
	lds	r18,adrtab+7		;LCD-Adresse 4. Zeilenmitte
	ldi	r16,9			;Offset fr Zeilenende
	add	r18,r16			;LCD-Adresse fr Zeilenende
	clr	r17			;LCD in Befehlsmodus setzen
	rcall	lcdw40			;Adresse setzen
	ldi	r18,4			;Dreieck-Symbol laden
	ldi	r17,1			;LCD in Datenmodus setzen
	rcall	lcdw40			;Zeichen ausgeben
dsy010:	ret
;
; ---------------
; 1-Wire Routinen
; ---------------
;
; Einzelnes Bit in 1-Wire-Puffer einschieben
; Register: r16,r17,x
;
bitsto:	ldi	xl,low(owbuff)
	ldi	xh,high(owbuff)		;Zeiger auf 1-Wire-Puffer
	ldi	r17,8			;8 Pufferbytes bearbeiten
	ror	r16			;neues Bit in Carry schieben
bits10:	ld	r16,x			;Pufferbyte holen
	ror	r16			;bertrag-Bit einschieben
	st	x+,r16			;und wieder speichern
	dec	r17			;alle Pufferbytes bearbeitet?
	brne	bits10			;nein -> Schleife
	ret
;
; 1-Wire, Reset und Presence-Test
; Register: r16
;	    r20   <- Busnummer
;	    Carry -> 0=ok, 1=Fehler
;
owres:	cpi	r20,0			;Busnummer=0?
	breq	ow0res			;ja -> bearbeiten
	cpi	r20,1			;Busnummer=1?
	breq	ow1res			;ja -> bearbeiten
	cpi	r20,2			;Busnummer=2?
	breq	ow2res			;ja -> bearbeiten
	cpi	r20,3			;Busnummer=3?
	breq	ow3res			;ja -> bearbeiten
	cpi	r20,4			;Busnummer=4?
	breq	ow4res			;ja -> bearbeiten
	cpi	r20,5			;Busnummer=5?
	breq	owrs10			;ja -> bearbeiten
	cpi	r20,6			;Busnummer=6?
	breq	owrs20			;ja -> bearbeiten
	rjmp	ow7res			;sonst Busnummer 7 bearbeiten
owrs10:	rjmp	ow5res
owrs20:	rjmp	ow6res
;
ow0res:	cbi	portc,0			;Strong-pullup auschalten
	sbi	ddrc,0			;Port auf Ausgang Low setzen
	rcall	w480us			;480s warten
	cbi	ddrc,0			;Port auf Eingang setzen
	rcall	w060us			;60s warten (Presence Start)
	in	r16,pinc		;Port lesen
	andi	r16,0x01		;Portpin selektieren
	breq	ow0s10			;Presence ok? ja -> weiter
	sec				;sonst Fehler
	ret
ow0s10:	rcall	w420us			;240s warten (Presence Ende)
	ret				;Reset ok
;
ow1res:	cbi	portc,1			;Strong-pullup auschalten
	sbi	ddrc,1			;Port auf Ausgang Low setzen
	rcall	w480us			;480s warten
	cbi	ddrc,1			;Port auf Eingang setzen
	rcall	w060us			;60s warten (Presence Start)
	in	r16,pinc		;Port lesen
	andi	r16,0x02		;Portpin selektieren
	breq	ow1s10			;Presence ok? ja -> weiter
	sec				;sonst Fehler
	ret
ow1s10:	rcall	w420us			;240s warten (Presence Ende)
	ret				;Reset ok
;
ow2res:	cbi	portc,2			;Strong-pullup auschalten
	sbi	ddrc,2			;Port auf Ausgang Low setzen
	rcall	w480us			;480s warten
	cbi	ddrc,2			;Port auf Eingang setzen
	rcall	w060us			;60s warten (Presence Start)
	in	r16,pinc		;Port lesen
	andi	r16,0x04		;Portpin selektieren
	breq	ow2s10			;Presence ok? ja -> weiter
	sec				;sonst Fehler
	ret
ow2s10:	rcall	w420us			;240s warten (Presence Ende)
	ret				;Reset ok
;
ow3res:	cbi	portc,3			;Strong-pullup auschalten
	sbi	ddrc,3			;Port auf Ausgang Low setzen
	rcall	w480us			;480s warten
	cbi	ddrc,3			;Port auf Eingang setzen
	rcall	w060us			;60s warten (Presence Start)
	in	r16,pinc		;Port lesen
	andi	r16,0x08		;Portpin selektieren
	breq	ow3s10			;Presence ok? ja -> weiter
	sec				;sonst Fehler
	ret
ow3s10:	rcall	w420us			;240s warten (Presence Ende)
	ret				;Reset ok
;
ow4res:	cbi	portd,4			;Strong-pullup auschalten
	sbi	ddrd,4			;Port auf Ausgang Low setzen
	rcall	w480us			;480s warten
	cbi	ddrd,4			;Port auf Eingang setzen
	rcall	w060us			;60s warten (Presence Start)
	in	r16,pind		;Port lesen
	andi	r16,0x10		;Portpin selektieren
	breq	ow4s10			;Presence ok? ja -> weiter
	sec				;sonst Fehler
	ret
ow4s10:	rcall	w420us			;240s warten (Presence Ende)
	ret				;Reset ok
;
ow5res:	cbi	portd,5			;Strong-pullup auschalten
	sbi	ddrd,5			;Port auf Ausgang Low setzen
	rcall	w480us			;480s warten
	cbi	ddrd,5			;Port auf Eingang setzen
	rcall	w060us			;60s warten (Presence Start)
	in	r16,pind		;Port lesen
	andi	r16,0x20		;Portpin selektieren
	breq	ow5s10			;Presence ok? ja -> weiter
	sec				;sonst Fehler
	ret
ow5s10:	rcall	w420us			;240s warten (Presence Ende)
	ret				;Reset ok
;
ow6res:	cbi	portd,6			;Strong-pullup auschalten
	sbi	ddrd,6			;Port auf Ausgang Low setzen
	rcall	w480us			;480s warten
	cbi	ddrd,6			;Port auf Eingang setzen
	rcall	w060us			;60s warten (Presence Start)
	in	r16,pind		;Port lesen
	andi	r16,0x40		;Portpin selektieren
	breq	ow6s10			;Presence ok? ja -> weiter
	sec				;sonst Fehler
	ret
ow6s10:	rcall	w420us			;240s warten (Presence Ende)
	ret				;Reset ok
;
ow7res:	cbi	portd,7			;Strong-pullup auschalten
	sbi	ddrd,7			;Port auf Ausgang Low setzen
	rcall	w480us			;480s warten
	cbi	ddrd,7			;Port auf Eingang setzen
	rcall	w060us			;60s warten (Presence Start)
	in	r16,pind		;Port lesen
	andi	r16,0x80		;Portpin selektieren
	breq	ow7s10			;Presence ok? ja -> weiter
	sec				;sonst Fehler
	ret
ow7s10:	rcall	w420us			;240s warten (Presence Ende)
	ret				;Reset ok
;
; 1-Wire, Schreiben eines Byte
; Register: r16,r18
;	    r19 zu sendendes Byte
;	    r20 Busnummer (0-7)
;
owwrit:	ldi	r18,8			;8 Bits ausgeben
owwr10:	ror	r19			;LSB in Carry schieben
	rcall	owbwri			;Bit ausgeben
	dec	r18			;alle Bits gesendet?
	brne	owwr10			;nein -> Schleife
	ret
;
; 1-Wire, Lesen eines Bytes
; Register: r16,r17,r18
;	    r19 -> gelesenes Byte
;	    r20 <- Busnummer (0-7)
;
owread:	ldi	r18,8			;8 Bits ausgeben
owre10:	rcall	owbrea			;Bit einlesen
	ror	r19			;Bit aus Carry einschieben
	dec	r18			;alle Bits gelesen?
	brne	owre10			;nein -> Schleife
	ret
;
; 1-Wire, Schreiben eines Bits
; Register: r16
;	    r20 <- Busnummer (0-7)
;	    Carry <- zu schreibendes Bit
;
owbwri:	mov	r16,r20			;Busnummer kopieren
	tst	r16			;Busnummer=0?
	breq	ow0wri			;ja -> bearbeiten
	dec	r16			;Busnummer=1?
	breq	ow1wri			;ja -> bearbeiten
	dec	r16			;Busnummer=2?
	breq	ow2wri			;ja -> bearbeiten
	dec	r16			;Busnummer=3?
	breq	ow3wri			;ja -> bearbeiten
	dec	r16			;Busnummer=4?
	breq	ow4wri			;ja -> bearbeiten
	dec	r16			;Busnummer=5?
	breq	ow5wri			;ja -> bearbeiten
	dec	r16			;Busnummer=6?
	breq	ow6wri			;ja -> bearbeiten
	rjmp	ow7wri			;sonst Busnummer 7 bearbeiten
;
ow0wri:	sbi	ddrc,0			;Port auf Ausgang Low setzen
	brcs	ow0w10			;1-Bit? ja -> weiter
	rcall	w060us			;60s warten (=Low)
	cbi	ddrc,0			;Port auf Eingang setzen
	rcall	w005us			;4,5s warten (Pause)
	ret
ow0w10:	rcall	w002us			;2s warten (=High)
	cbi	ddrc,0			;Port auf Eingang setzen
	rcall	w060us			;60s warten
	ret
;
ow1wri:	sbi	ddrc,1			;Port auf Ausgang Low setzen
	brcs	ow1w10			;1-Bit? ja -> weiter
	rcall	w060us			;60s warten (=Low)
	cbi	ddrc,1			;Port auf Eingang setzen
	rcall	w005us			;4,5s warten (Pause)
	ret
ow1w10:	rcall	w002us			;2s warten (=High)
	cbi	ddrc,1			;Port auf Eingang setzen
	rcall	w060us			;60s warten
	ret
;
ow2wri:	sbi	ddrc,2			;Port auf Ausgang Low setzen
	brcs	ow2w10			;1-Bit? ja -> weiter
	rcall	w060us			;60s warten (=Low)
	cbi	ddrc,2			;Port auf Eingang setzen
	rcall	w005us			;4,5s warten (Pause)
	ret
ow2w10:	rcall	w002us			;2s warten (=High)
	cbi	ddrc,2			;Port auf Eingang setzen
	rcall	w060us			;60s warten
	ret
;
ow3wri:	sbi	ddrc,3			;Port auf Ausgang Low setzen
	brcs	ow3w10			;1-Bit? ja -> weiter
	rcall	w060us			;60s warten (=Low)
	cbi	ddrc,3			;Port auf Eingang setzen
	rcall	w005us			;4,5s warten (Pause)
	ret
ow3w10:	rcall	w002us			;2s warten (=High)
	cbi	ddrc,3			;Port auf Eingang setzen
	rcall	w060us			;60s warten
	ret
;
ow4wri:	sbi	ddrd,4			;Port auf Ausgang Low setzen
	brcs	ow4w10			;1-Bit? ja -> weiter
	rcall	w060us			;60s warten (=Low)
	cbi	ddrd,4			;Port auf Eingang setzen
	rcall	w005us			;4,5s warten (Pause)
	ret
ow4w10:	rcall	w002us			;2s warten (=High)
	cbi	ddrd,4			;Port auf Eingang setzen
	rcall	w060us			;60s warten
	ret
;
ow5wri:	sbi	ddrd,5			;Port auf Ausgang Low setzen
	brcs	ow5w10			;1-Bit? ja -> weiter
	rcall	w060us			;60s warten (=Low)
	cbi	ddrd,5			;Port auf Eingang setzen
	rcall	w005us			;4,5s warten (Pause)
	ret
ow5w10:	rcall	w002us			;2s warten (=High)
	cbi	ddrd,5			;Port auf Eingang setzen
	rcall	w060us			;60s warten
	ret
;
ow6wri:	sbi	ddrd,6			;Port auf Ausgang Low setzen
	brcs	ow6w10			;1-Bit? ja -> weiter
	rcall	w060us			;60s warten (=Low)
	cbi	ddrd,6			;Port auf Eingang setzen
	rcall	w005us			;4,5s warten (Pause)
	ret
ow6w10:	rcall	w002us			;2s warten (=High)
	cbi	ddrd,6			;Port auf Eingang setzen
	rcall	w060us			;60s warten
	ret
;
ow7wri:	sbi	ddrd,7			;Port auf Ausgang Low setzen
	brcs	ow7w10			;1-Bit? ja -> weiter
	rcall	w060us			;60s warten (=Low)
	cbi	ddrd,7			;Port auf Eingang setzen
	rcall	w005us			;4,5s warten (Pause)
	ret
ow7w10:	rcall	w002us			;2s warten (=High)
	cbi	ddrd,7			;Port auf Eingang setzen
	rcall	w060us			;60s warten
	ret
;
; 1-Wire, Lesen eines Bits
; Register: r16,r17
;	    r20   <- Busnummer (0-7)
;	    Carry -> gelesenes Bit
;
owbrea:	mov	r16,r20			;Busnummer kopieren
	tst	r16			;Busnummer=0?
	breq	ow0rea			;ja -> bearbeiten
	dec	r16			;Busnummer=1?
	breq	ow1rea			;ja -> bearbeiten
	dec	r16			;Busnummer=2?
	breq	ow2rea			;ja -> bearbeiten
	dec	r16			;Busnummer=3?
	breq	ow3rea			;ja -> bearbeiten
	dec	r16			;Busnummer=4?
	breq	ow4rea			;ja -> bearbeiten
	dec	r16			;Busnummer=5?
	breq	owbr10			;ja -> bearbeiten
	dec	r16			;Busnummer=6?
	breq	owbr20			;ja -> bearbeiten
	rjmp	ow7rea			;sonst Busnummer 7 bearbeiten
owbr10:	rjmp	ow5rea
owbr20:	rjmp	ow6rea
;
ow0rea:	sbi	ddrc,0			;Port auf Ausgang Low setzen
	rcall	w002us			;2s warten
	cbi	ddrc,0			;Port auf Eingang setzen
	rcall	w005us			;4,5s warten (Pause)
	rcall	w005us			;4,5s warten (Pause)
	in	r17,pinc		;Port lesen
	andi	r17,0x01		;Portpin selektieren
	clc				;Carry lschen (=0)
	breq	ow0r10			;Port=0? ja -> weiter
	sec				;sonst Carry setzen (=1)
ow0r10:	rcall	w050us			;50s warten
	ret
;
ow1rea:	sbi	ddrc,1			;Port auf Ausgang Low setzen
	rcall	w002us			;2s warten
	cbi	ddrc,1			;Port auf Eingang setzen
	rcall	w005us			;4,5s warten (Pause)
	rcall	w005us			;4,5s warten (Pause)
	in	r17,pinc		;Port lesen
	andi	r17,0x02		;Portpin selektieren
	clc				;Carry lschen (=0)
	breq	ow1r10			;Port=0? ja -> weiter
	sec				;sonst Carry setzen (=1)
ow1r10:	rcall	w050us			;50s warten
	ret
;
ow2rea:	sbi	ddrc,2			;Port auf Ausgang Low setzen
	rcall	w002us			;2s warten
	cbi	ddrc,2			;Port auf Eingang setzen
	rcall	w005us			;4,5s warten (Pause)
	rcall	w005us			;4,5s warten (Pause)
	in	r17,pinc		;Port lesen
	andi	r17,0x04		;Portpin selektieren
	clc				;Carry lschen (=0)
	breq	ow2r10			;Port=0? ja -> weiter
	sec				;sonst Carry setzen (=1)
ow2r10:	rcall	w050us			;50s warten
	ret
;
ow3rea:	sbi	ddrc,3			;Port auf Ausgang Low setzen
	rcall	w002us			;2s warten
	cbi	ddrc,3			;Port auf Eingang setzen
	rcall	w005us			;4,5s warten (Pause)
	rcall	w005us			;4,5s warten (Pause)
	in	r17,pinc		;Port lesen
	andi	r17,0x08		;Portpin selektieren
	clc				;Carry lschen (=0)
	breq	ow3r10			;Port=0? ja -> weiter
	sec				;sonst Carry setzen (=1)
ow3r10:	rcall	w050us			;50s warten
	ret
;
ow4rea:	sbi	ddrd,4			;Port auf Ausgang Low setzen
	rcall	w002us			;2s warten
	cbi	ddrd,4			;Port auf Eingang setzen
	rcall	w005us			;4,5s warten (Pause)
	rcall	w005us			;4,5s warten (Pause)
	in	r17,pind		;Port lesen
	andi	r17,0x10		;Portpin selektieren
	clc				;Carry lschen (=0)
	breq	ow4r10			;Port=0? ja -> weiter
	sec				;sonst Carry setzen (=1)
ow4r10:	rcall	w050us			;60s warten
	ret
;
ow5rea:	sbi	ddrd,5			;Port auf Ausgang Low setzen
	rcall	w002us			;2s warten
	cbi	ddrd,5			;Port auf Eingang setzen
	rcall	w005us			;4,5s warten (Pause)
	rcall	w005us			;4,5s warten (Pause)
	in	r17,pind		;Port lesen
	andi	r17,0x20		;Portpin selektieren
	clc				;Carry lschen (=0)
	breq	ow5r10			;Port=0? ja -> weiter
	sec				;sonst Carry setzen (=1)
ow5r10:	rcall	w050us			;60s warten
	ret
;
ow6rea:	sbi	ddrd,6			;Port auf Ausgang Low setzen
	rcall	w002us			;2s warten
	cbi	ddrd,6			;Port auf Eingang setzen
	rcall	w005us			;4,5s warten (Pause)
	rcall	w005us			;4,5s warten (Pause)
	in	r17,pind		;Port lesen
	andi	r17,0x40		;Portpin selektieren
	clc				;Carry lschen (=0)
	breq	ow6r10			;Port=0? ja -> weiter
	sec				;sonst Carry setzen (=1)
ow6r10:	rcall	w050us			;60s warten
	ret
;
ow7rea:	sbi	ddrd,7			;Port auf Ausgang Low setzen
	rcall	w002us			;2s warten
	cbi	ddrd,7			;Port auf Eingang setzen
	rcall	w005us			;4,5s warten (Pause)
	rcall	w005us			;4,5s warten (Pause)
	in	r17,pind		;Port lesen
	andi	r17,0x80		;Portpin selektieren
	clc				;Carry lschen (=0)
	breq	ow7r10			;Port=0? ja -> weiter
	sec				;sonst Carry setzen (=1)
ow7r10:	rcall	w050us			;60s warten
	ret
;
; 1-Wire, Strong Pullup, Bus auf High legen whrend Temperaturmessung
; Register: r16
;	    r20 <- Busnummer
;
owstpu:	cpi	r20,0			;Busnummer=0?
	breq	ow0stp			;ja -> bearbeiten
	cpi	r20,1			;Busnummer=1?
	breq	ow1stp			;ja -> bearbeiten
	cpi	r20,2			;Busnummer=2?
	breq	ow2stp			;ja -> bearbeiten
	cpi	r20,3			;Busnummer=3?
	breq	ow3stp			;ja -> bearbeiten
	cpi	r20,4			;Busnummer=4?
	breq	ow4stp			;ja -> bearbeiten
	cpi	r20,5			;Busnummer=5?
	breq	ow5stp			;ja -> bearbeiten
	cpi	r20,6			;Busnummer=6?
	breq	ow6stp			;ja -> bearbeiten
	rjmp	ow7stp			;sonst Busnummer 7 bearbeiten
;
ow0stp:	sbi	portc,0			;Portausgang auf High setzen
	sbi	ddrc,0			;Port auf Ausgang umschalten
	ret
;
ow1stp:	sbi	portc,1			;Portausgang auf High setzen
	sbi	ddrc,1			;Port auf Ausgang umschalten
	ret
;
ow2stp:	sbi	portc,2			;Portausgang auf High setzen
	sbi	ddrc,2			;Port auf Ausgang umschalten
	ret
;
ow3stp:	sbi	portc,3			;Portausgang auf High setzen
	sbi	ddrc,3			;Port auf Ausgang umschalten
	ret
;
ow4stp:	sbi	portd,4			;Portausgang auf High setzen
	sbi	ddrd,4			;Port auf Ausgang umschalten
	ret
;
ow5stp:	sbi	portd,5			;Portausgang auf High setzen
	sbi	ddrd,5			;Port auf Ausgang umschalten
	ret
;
ow6stp:	sbi	portd,6			;Portausgang auf High setzen
	sbi	ddrd,6			;Port auf Ausgang umschalten
	ret
;
ow7stp:	sbi	portd,7			;Portausgang auf High setzen
	sbi	ddrd,7			;Port auf Ausgang umschalten
	ret
;
; CRC-Berechnung fr 1-Wire-Devices fr 8 Byte im 1-Wire-Puffer, nach
; Application Note 27 von Dallas/Maxim
; Register: r10,r11,r16,r18,r19,x
;	    r17 -> ermittelter CRC-Wert
;
crc:	ldi	xl,low(owbuff+8)
	ldi	xh,high(owbuff+8)	;Zeiger auf 1-Wire-Puffer-Ende
	ldi	r19,8			;8 Bytes berechnen
	ldi	r16,0x18		;Konstante fr Berechnung
	mov	r10,r16			;bereitlegen
	clr	r17			;CRC-Startwert=0 setzen
;
crc100:	ld	r18,-x			;Byte (LSB) aus Puffer holen
	mov	r11,r18			;Byte zwischenspeichern
	ldi	r16,8			;8 Verschiebungen
crc200:	eor	r18,r17			;CRC berechnen
	ror	r18			;Ergebnis in Carry schieben
	mov	r18,r17			;letzten CRC-Wert holen
	brcc	crc300			;war Ergebnis=0? ja -> weiter
	eor	r18,r10			;sonst CRC-Wert updaten
crc300:	ror	r18			;neuen CRC-Wert positionieren
	mov	r17,r18			;und speichern
	mov	r18,r11			;restliche Bits holen
	bst	r18,0			;LSB sichern, nchste Bitpos.,
	ror	r18			;gesichertes Bit als MSB ein-
	bld	r18,7			;setzen (Ersatz fr RR-Befehl)
	mov	r11,r18			;und Wert zwischenspeichern
	dec	r16			;alle Verschiebungen erledigt?
	brne	crc200			;nein -> Schleife
	dec	r19			;alle Bytes bearbeitet?
	brne	crc100			;nein -> Schleife
	ret
;
; ---------------
; RS-232 Routinen
; ---------------
;
; Alarmausgabe ber RS-232
; Register: r16
;	    r19 <- Alarmwert (0/1)
;	    r20 <- Alarmnummer (0-3)
;
rsaout:	mov	r16,r20			;aktuelle Alarmnummer holen
	inc	r16			;Nummer korrigieren
	ori	r16,0x40		;in ASCII-Zeichen wandeln
	rcall	rssend			;Zeichen senden
	ldi	r16,':'			;Doppelpunkt laden
	rcall	rssend			;Zeichen senden
	ldi	r16,'0'			;Zeichen '0' laden
	add	r16,r19			;Alarmwert addieren
	rcall	rssend			;Zeichen senden
	ldi	r16,cr			;CR-Zeichen laden
	rcall	rssend			;Zeichen senden
	ret
;
; USART Senden (RS-232)
; Register: r16 <- zu sendendes Zeichen
;
rssend:	sbis	ucsra,udre		;Sendepuffer leer?
	rjmp	rssend			;nein -> warten
	out	udr,r16			;sonst Byte senden
	ret
;
; ------------
; LCD-Routinen
; ------------
;
; LCD-Ausgabe eines Textes unter Bercksichtigung der Men-Sprache
; Register: r16,r17,r18,z
;	    r19 <- Textnummer
;	    r20 <- Zeichenzhler (wird bei jedem Zeichen dekrementiert)
;
lcdtxt:	ldi	r17,1			;LCD in Datenmodus setzen
	lds	r16,lang		;Men-Sprache holen
	tst	r16			;Sprache Englisch?
	brne	lcdt10			;nein -> weiter
	ldi	zl,low(entext*2)	;sonst
	ldi	zh,high(entext*2)	;Zeiger auf englische Texte
	rjmp	lcdt20
;
lcdt10:	ldi	zl,low(getext*2)
	ldi	zh,high(getext*2)	;Zeiger auf deutsche Texte
;
lcdt20:	lpm	r16,z+			;Zeichen holen
	cpi	r16,0xfe		;Text-Kennung gefunden?
	brne	lcdt20			;nein -> weiter suchen
	lpm	r16,z+			;Zeichen holen
	cp	r16,r19			;Text gefunden?
	breq	lcdt30			;ja -> ausgeben
	cpi	r16,0xfe		;Endezeichen gefunden?
	brne	lcdt20			;nein -> weiter suchen
	ret				;sonst Ende
;
lcdt30:	lpm	r18,z+			;Zeichen holen
	cpi	r18,0xff		;Endezeichen?
	brne	lcdt40			;nein -> weiter
	ret				;sonst Ende
lcdt40:	rcall	lcdw40			;Zeichen ausgeben
	dec	r20			;Zeichenzhler-1
	rjmp	lcdt30			;nchstes Zeichen bearbeiten
;
; LCD-Ausgabe, Auffllen einer Zeile mit Leerzeichen
; Register: r16,r17,r18
;	    r20 <- Anzahl der Zeichen bis Zeilenende
;
lcdlin:	ldi	r17,1			;LCD in Datenmodus setzen
lcdl10:	tst	r20			;Zeichenzhler=0?
	brne	lcdl20			;nein -> weiter
	ret				;sonst Ende
;
lcdl20:	ldi	r18,' '			;Leerzeichen-Code laden
	rcall	lcdw40			;ausgeben
	dec	r20			;Zeichenzhler-1
	rjmp	lcdl10			;Schleife
;
; LCD-Ausgabe eines 4-stelligen Dezimalwertes
; Register: r16,r17,r18,r19,r21,x
;	    r0,r1 <- 16-Bit-Wert
;	    r20   <- Zeichenzhler (bei jedem Zeichen dekrementiert)
;
lcddez:	clr	r21			;Tausender=0 setzen
	movw	r18,r0			;Wert kopieren
	ldi	r16,low(1000)
	ldi	r17,high(1000)
lcdd10:	sub	r18,r16			;Wert-1000
	sbc	r19,r17			;Ergebnis negativ?
	brmi	lcdd20			;ja -> weiter
	movw	r0,r18			;sonst Ergebnis kopieren
	inc	r21			;Tausender zhlen
	rjmp	lcdd10			;wiederholen
;
lcdd20:	tst	r21			;Tausender=0?
	breq	lcdd30			;ja -> weiter
	ori	r21,0x30		;Tausender in ASCII wandeln
	rjmp	lcdd40			;und speichern
lcdd30:	ldi	r21,' '			;Leerzeichen laden
lcdd40:	sts	dezval,r21		;speichern
	clr	r21			;Hunderter=0 setzen
	movw	r18,r0			;letzten Wert kopieren
	ldi	r16,100
	clr	r17
lcdd50:	sub	r18,r16			;Wert-100
	sbc	r19,r17			;Ergebnis negativ?
	brmi	lcdd60			;ja -> weiter
	movw	r0,r18			;sonst Ergebnis kopieren
	inc	r21			;Hunderter zhlen
	rjmp	lcdd50			;wiederholen
;
lcdd60:	ori	r21,0x30		;Hunderter in ASCII wandeln
	sts	dezval+1,r21		;und speichern
	clr	r21			;Zehner=0 setzen
	movw	r18,r0			;letzten Wert kopieren
	ldi	r16,10
	clr	r17
lcdd70:	sub	r18,r16			;Wert-10
	sbc	r19,r17			;Ergebnis negativ?
	brmi	lcdd80			;ja -> weiter
	movw	r0,r18			;sonst Ergebnis kopieren
	inc	r21			;Zehner zhlen
	rjmp	lcdd70			;wiederholen
;
lcdd80:	ori	r21,0x30		;Zehner in ASCII wandeln
	sts	dezval+2,r21		;und speichern
	mov	r21,r0			;Restwert (Einer) holen
	ori	r21,0x30		;Einer in ASCII wandeln
	sts	dezval+3,r21		;und speichern
;
	ldi	xl,low(dezval)
	ldi	xh,high(dezval)		;Zeiger auf Dezimalwert
	ldi	r19,4			;4 Zeichen ausgeben
	ldi	r17,1			;LCD in Datenmodus setzen
lcdd90:	ld	r18,x+			;Zeichen holen
	rcall	lcdw40			;ausgeben
	dec	r20			;Zeichenzhler-1
	dec	r19			;alle Zeichen ausgegeben?
	brne	lcdd90			;nein -> Schleife
	ret
;
; LCD-Ausgabe eines Zeichens im 4-Bit Mode
; Register: r16
;	    r17 <- 0=Befehlsbyte, 1=Datenbyte
;	    r18 <- LCD-Daten oder -Befehl
;
lcdout:	tst	r17			;Datenbyte?
	brne	lcdo10			;ja -> weiter
	cbi	portc,4			;LCD in Befehlsmodus setzen
	rjmp	lcdo20
lcdo10:	sbi	portc,4			;LCD in Datenmodus setzen
lcdo20:	mov	r16,r18			;Datenbyte kopieren
	ror	r18			;oberes Nibble an die richtige
	ror	r18			;Position schieben
	andi	r18,0x3c		;Nibble filtern
	ori	r18,0x03		;Pullup fr Al1 und Al2 setzen
	out	portb,r18		;Daten/Befehlsbyte ausgeben
	sbi	portc,5			;E auf High setzen
	cbi	portc,5			;E wieder auf Low setzen
;
	mov	r18,r16			;Datenbyte kopieren
	rol	r18			;unteres Nibble an die richtige
	rol	r18			;Position schieben
	andi	r18,0x3c		;Nibble filtern
	ori	r18,0x03		;Pullup fr Al1 und Al2 setzen
	out	portb,r18		;Daten/Befehlsbyte ausgeben
	sbi	portc,5			;E auf High setzen
	cbi	portc,5			;E wieder auf Low setzen
	ret
;
; LCD-Ausgabe eines Zeichens im 4-Bit Mode und Warteschleife 40s
; Register: r16
;	    r17 <- LCD-Daten/Befehlsbyte
;	    r18 <- 0=Befehlsbyte, 1=Datenbyte
;
lcdw40:	rcall	lcdout			;Zeichen ausgeben
	rjmp	w040us			;Warteschleife 40s
;
; LCD-Ausgabe eines Zeichens im 8-Bit Mode
; Register: r16
;	    r17 <- 0=Befehlsbyte, 1=Datenbyte
;	    r18 <- LCD-Daten oder -Befehl
;
lcd8bt:	tst	r17			;Datenbyte?
	brne	lcd810			;ja -> weiter
	cbi	portc,4			;LCD in Befehlsmodus setzen
	rjmp	lcd820
lcd810:	sbi	portc,4			;LCD in Datenmodus setzen
lcd820:	ror	r18			;oberes Nibble an die richtige
	ror	r18			;Position schieben
	andi	r18,0x3c		;Nibble filtern
	ori	r18,0x03		;Pullup fr Al1 und Al2 setzen
	out	portb,r18		;Daten/Befehlsbyte ausgeben
	sbi	portc,5			;E auf High setzen
	cbi	portc,5			;E wieder auf Low setzen
	ret
;
; --------------
; Warteschleifen
; --------------
;
; Warteschleife ca. 50ms
; Register: r15,r16
;
w050ms:	clr	r15			;Zeitkonstante uere Schleife
	clr	r16			;Zeitkonstante innere Schleife
w050mx:	dec	r16
	brne	w050mx			;innere Schleife bearbeiten
	dec	r15
	brne	w050mx			;uere Schleife bearbeiten
	ret	
;
; Warteschleife 4,1ms
; Register: r15,r16
;
w004ms:	ldi	r16,22			;Zeitkonstante uere Schleife
	mov	r15,r16			;kopieren
	ldi	r16,73			;Zeitkonstante innere Schleife
	rjmp	w050mx
;
; Warteschleife 480s
; Register: r15,r16
;
w480us:	ldi	r16,3			;Zeitkonstante uere Schleife
	mov	r15,r16			;kopieren
	ldi	r16,123			;Zeitkonstante innere Schleife
	rjmp	w050mx	
;
; Warteschleife 420s
; Register: r15,r16
;
w420us:	ldi	r16,3			;Zeitkonstante uere Schleife
	mov	r15,r16			;kopieren
	ldi	r16,42			;Zeitkonstante innere Schleife
	rjmp	w050mx
;
; Warteschleife 100s
; Register: r15,r16
;
w100us:	ldi	r16,1			;Zeitkonstante uere Schleife
	mov	r15,r16			;kopieren
	ldi	r16,129			;Zeitkonstante innere Schleife
	rjmp	w050mx
;
; Warteschleife 60s
; Register: r16
;
w060us:	ldi	r16,76			;Zeitkonstante laden
w060xx:	dec	r16
	brne	w060xx			;Schleife bearbeiten
	ret
;
; Warteschleife 50s
; Register: r16
;
w050us:	ldi	r16,65			;Zeitkonstante laden
w050xx:	dec	r16
	brne	w050xx			;Schleife bearbeiten
	ret
;
; Warteschleife 40s
; Register: r16
;
w040us:	ldi	r16,49			;Zeitkonstante laden
w040xx:	dec	r16
	brne	w040xx			;Schleife bearbeiten
	ret
;
; Warteschleife 4,5s
; Register: r16
;
w005us:	ldi	r16,4			;Zeitkonstante laden
w005xx:	dec	r16
	brne	w005xx			;Schleife bearbeiten
	ret
;
; Warteschleife 2s
; Register: keine
;
w002us:	nop
	ret
;
; ---------------
; EEPROM-Routinen
; ---------------
;
; Lesen eines Bytes aus dem EEPROM
; Register: r16 <- EEPROM-Adresse L
;	    r17 <- EEPROM-Adresse H
;	    r18 -> EEPROM-Daten
;
eeread:	sbic	eecr,eewe		;luft Schreibzyklus?
	rjmp	eeread			;ja - warten
	out	eearl,r16		;EEPROM-Adresse L setzen
	out	eearh,r17		;EEPROM-Adresse H setzen
	sbi	eecr,eere		;EEPROM lesen aktivieren
	in	r18,eedr		;Daten lesen
	ret
;
; Schreiben eines Bytes in das EEPROM
; Register: r16 <- EEPROM-Adresse L
;	    r17 <- EEPROM-Adresse H
;	    r18 <- EEPROM-Daten
;
eewrit:	sbic	eecr,eewe		;luft Schreibzyklus?
	rjmp	eewrit			;ja -> warten
	out	eearh,r17		;EEPROM-Adresse H setzen
	out	eearl,r16		;EEPROM-Adresse L setzen
	out	eedr,r18		;EEPROM-Datenbyte setzen
	sbi	eecr,eemwe		;Masterwrite-Bit setzen
	sbi	eecr,eewe		;Write-Bit setzen
	ret
;
; ----------------
; Diverse Routinen
; ----------------
;
; Anzahl der Sensoren ermitteln und speichern
; Register: r16,r17,r18,z
;
getsen:	ldi	zl,low(senstb)
	ldi	zh,high(senstb)		;Zeiger auf Sensortabelle
	ldi	r18,maxsen		;16 Werte durchsuchen
	clr	r17			;Sensorzhler=0 setzen
gets10:	ld	r16,z+			;Wert aus Tabelle lesen
	cpi	r16,8			;gltiger Wert?
	brcc	gets20			;nein -> weiter
	inc	r17			;sonst Sensorzhler erhhen
gets20:	dec	r18			;alle Sensoren bearbeitet?
	brne	gets10			;nein -> Schleife
	sts	sensrs,r17		;Anzahl der Sensoren speichern
	ret
;
; Sensorbelegung ermitteln und speichern
; Register: r16,r17,r18,r19,x,z
;
getmap:	ldi	r19,0			;Busnummer=0 setzen
	ldi	xl,low(senmap)
	ldi	xh,high(senmap)		;Zeiger auf Sensorbelegung
getm10:	ldi	zl,low(senstb)
	ldi	zh,high(senstb)		;Zeiger auf Sensortabelle
	ldi	r18,maxsen		;16 Werte durchsuchen
	ldi	r17,'0'			;Sensorzhler=0 setzen (ASCII)
getm20:	ld	r16,z+			;Wert aus Tabelle lesen
	cp	r16,r19			;Sensor am aktuellen Bus?
	brne	getm30			;nein -> weiter
	inc	r17			;sonst Sensorzhler erhhen
getm30:	dec	r18			;alle Tabellenpltze gelesen?
	brne	getm20			;nein -> Schleife
	cpi	r17,'9'+1		;mehr als 9 Sensoren am Bus?
	brcs	getm40			;nein -> weiter
	ldi	r16,7			;sonst Zahl in Buchstaben
	add	r17,r16			;wandeln: 10..16 -> A..G
getm40:	st	x+,r17			;Sensoranzahl speichern
	inc	r19			;nchsten Bus testen
	cpi	r19,8			;alle Busse getestet?
	brne	getm10			;nein -> Schleife
	ret
;
; Sensordaten im EEPROM speichern
; Register: r16,r17,r18,r19,r20,x
;
savsen:	ldi	r20,maxsen		;Anzahl der Sensoren holen (16)
	ldi	r16,low(esenid)
	ldi	r17,high(esenid)	;EEPROM-Adresse der Sensor-IDs
	ldi	xl,low(sensid)
	ldi	xh,high(sensid)		;Zeiger auf Sensor-IDs im RAM
savei1:	ldi	r19,8			;jeweils 8 ID-Bytes
savei2:	ld	r18,x+			;Byte aus RAM holen
	rcall	eewrit			;und im EEPROM speichern
	inc	r16			;nchste EEPROM-Adresse L
	brne	savei3			;bertrag? nein -> weiter
	inc	r17			;nchste EEPROM-Adresse H
savei3:	dec	r19			;alle 8 Bytes gelesen?
	brne	savei2			;nein -> Schleife
	dec	r20			;alle Sensoren gelesen?
	brne	savei1			;nein -> Schleife
;
	ldi	r20,maxsen		;Anzahl der Sensoren holen (16)
	ldi	r16,low(esentb)
	ldi	r17,high(esentb)	;EEPROM-Adresse der Sensor-Tab.
	ldi	xl,low(senstb)
	ldi	xh,high(senstb)		;Zeiger auf Sensor-Tab. im RAM
savet1:	ld	r18,x+			;Byte aus RAM holen
	rcall	eewrit			;und im EEPROM speichern
	inc	r16			;nchste EEPROM-Adresse L
	brne	savet2			;bertrag? nein -> weiter
	inc	r17			;nchste EEPROM-Adresse H
savet2:	dec	r20			;alle Sensoren gelesen?
	brne	savet1			;nein -> Schleife
	ret
;
; 16-Bit Interrupt-Zhler auf Anzeige-Timeout setzen (60s)
; Register: r16,r17
;
setinc:	ldi	r16,low(setrtn)
	ldi	r17,high(setrtn)
	sts	intc16,r16		;Interrupt-Zhler auf Rckkehr-
	sts	intc16+1,r17		;Zeit in Automatik-Modus setzen
	ret
;
; ------------------
; Interrupt-Routinen
; ------------------
;
; Timer1 Interrupt, wird alle 50ms aufgerufen
; Register: isreg,itemp1,itemp2,y
;
; Abfrage der Alarmeingnge
;
timer1:	in	isreg,sreg		;SREG sichern
	ldi	yl,low(alstat)
	ldi	yh,high(alstat)		;Zeiger auf Alarmstatus
	ld	itemp2,y		;Alarmstatus holen
	in	itemp1,pinb		;Alarmeingnge 1 und 2 lesen
	com	itemp1			;und invertieren
	bst	itemp1,0		;Alarmbit 1 holen
	bld	itemp2,0		;und in Status einsetzen
	st	y+,itemp2		;Alarmstatus speichern
	ld	itemp2,y		;nchsten Alarmstatus holen
	bst	itemp1,1		;Alarmbit 2 holen
	bld	itemp2,0		;und in Status einsetzen
	st	y+,itemp2		;Alarmstatus speichern
	ld	itemp2,y		;nchsten Alarmstatus holen
	in	itemp1,pind		;Alarmeingnge 3 und 4 lesen
	com	itemp1			;und invertieren
	bst	itemp1,2		;Alarmbit 3 holen
	bld	itemp2,0		;und in Status einsetzen
	st	y+,itemp2		;Alarmstatus speichern
	ld	itemp2,y		;nchsten Alarmstatus holen
	bst	itemp1,3		;Alarmbit 2 holen
	bld	itemp2,0		;und in Status einsetzen
	st	y+,itemp2		;Alarmstatus speichern
;
; Steuerung der Alarme (Flags, Alarmzhler)
;
	ldi	yl,low(alstat)
	ldi	yh,high(alstat)		;Zeiger auf Alarmstatus
	lds	itemp2,altime		;Alarm-Zeit laden
	mov	itemp1,itemp2		;und vorbereiten
tim100:	ld	itemp2,y		;Alarmstatus holen
	andi	itemp2,0b00000011	;nur Statusbits 0/1 bewerten
	cpi	itemp2,0b00000000	;Status gleich geblieben?
	breq	tim130			;ja -> berspringen
	cpi	itemp2,0b00000011	;Status gleich geblieben?
	breq	tim130			;ja -> berspringen
	ld	itemp2,y		;Alarmstatus nochmals holen
	bst	itemp2,2		;Bearbeitungs-Bit gesetzt?
	brts	tim110			;ja -> weiter
	sbr	itemp2,0b00000100	;sonst Bearbeitungs-Flag setzen
	st	y,itemp2		;Alarmstatus wieder speichern
	std	y+4,itemp1		;Alarmzhler neu laden
tim110:	ldd	itemp2,y+4		;Alarmzhler holen
	cpi	itemp2,0		;Zhler abgelaufen?
	brne	tim120			;nein -> berspringen
	ld	itemp2,y		;Alarmstatus nochmals holen
	bst	itemp2,0		;aktuelles Statusbit holen
	bld	itemp2,1		;und als neuen Status setzen
	cbr	itemp2,0b00000100	;Bearbeitungs-Flag lschen
	sbr	itemp2,0b00001000	;Melde-Flag setzen
	bld	itemp2,4		;Melde-Status setzen
	st	y,itemp2		;neuen Alarmstatus speichern
	rjmp	tim140
tim120:	dec	itemp2			;sonst Zhler vermindern
	std	y+4,itemp2		;und wieder speichern
	rjmp	tim140
tim130:	ld	itemp2,y		;Alarmstatus nochmals holen
	cbr	itemp2,0b00000100	;Bearbeitungs-Flag lschen
	st	y,itemp2		;neuen Alarmstatus speichern
tim140:	adiw	yl,1			;Zeiger erhhen
	cpi	yl,low(alstat+4)	;alle Werte bearbeitet?
	brne	tim100			;nein -> Schleife
;
; Steuerung des Tasters, kurze und lange Bettigung
;
	bst	flags,3			;letzten Taster-Status holen
	bld	itemp2,0		;in Arbeitsregister legen
	in	itemp1,pind		;Tastereingang lesen
	com	itemp1			;invertieren
	bst	itemp1,0		;Taster-Bit kopieren
	bld	flags,3			;als neuen Status speichern
	eor	itemp2,itemp1		;Statusnderung?
	andi	itemp2,0x01		;Ergebnis filtern
	brne	tim300			;Statusnderung? ja -> weiter
	bld	flags,4			;sonst Entprellt-Status setzen
	lds	itemp2,keycnt		;Taster-Timeout-Zhler holen
	brts	tim200			;Gedrckt-Status? ja -> weiter
	cbr	flags,0b11110000	;sonst alle Taster-Flags lsch.
	ldi	itemp2,keytio		;und Timeout-Zhler neu setzen
	rjmp	tim220
tim200:	tst	itemp2			;Timeout-Zhler=0?
	brne	tim210			;nein -> weiter, sonst
	sbr	flags,0b01000000	;Flag: Taster lang gedrckt
	rjmp	tim220
tim210:	dec	itemp2			;Timeout-Zhler vermindern
tim220:	sts	keycnt,itemp2		;neuen Zhlerstand speichern
;
; 16-Bit Interrupt-Zhler vermindern und Interrupt-Flag setzen
;
tim300:	lds	itemp2,intc16		;Interrupt-Zhler L holen
	lds	itemp1,intc16+1		;Interrupt-Zhler H holen
	or	itemp1,itemp2		;verknpfen, Zhlerstand=0?
	breq	tim500			;ja -> Ende
;
	subi	itemp2,1		;Interrupt-Zhler L-1
	sts	intc16,itemp2		;Ergebnis L speichern
	lds	itemp2,intc16+1		;Interrupt-Zhler H holen
	sbci	itemp2,0		;bertrag subtrahieren
	sts	intc16+1,itemp2		;Ergebnis H speichern
;
tim500:	sbr	flags,0b00000001	;Interrupt-Flag setzen
	out	sreg,isreg		;SREG wieder herstellen
	reti
;
; ------------------
; Tabellen und Texte
; ------------------
;
; LCD-Adress-Tabelle
;
lcdtab:	.db	0x80,0xc0,0x94,0xd4	;LCD-Adresse fr Werte 1-4
	.db	0x8a,0xca,0x9e,0xde	;LCD-Adresse fr Werte 5-8
;
; LCD Sonderzeichen
;
xchars:	.db	2,6,0x0e,0x1e,0x0e,6,2,0	     ;Pfeil links
	.db	0x10,0x18,0x1c,0x1e,0x1c,0x18,0x10,0 ;Pfeil rechts
	.db	0x1f,0x11,0x11,0x11,0x11,0x11,0x1f,0 ;offen
	.db	0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,0 ;geschlossen
	.db	0,0,1,3,7,0x0f,0x1f,0		     ;Dreieck-Symbol
;
; Deutsche Texte
;
getext:	.db	0xfe,0,"Sensormodul2 v1.00",0xff,0xff
	.db	0xfe,1,"Sensoren: ",0xff,0xff
	.db	0xfe,2,"Belegung: ",0xff,0xff
	.db	0xfe,3,"Modus   : ",0xff,0xff
	.db	0xfe,4,"Direkt",0xff,0xff
	.db	0xfe,5,"Speicher",0xff,0xff
	.db	0xfe,6,1," ",0xff,0xff		;Pfeil rechts
	.db	0xfe,7," ",0,0xff,0xff		;Pfeil links
	.db	0xfe,8,"Lang ~ ",0xe1,"ndern",0xff
	.db	0xfe,9,"Kurz ~ weiter",0xff
	.db	0xfe,10,"Kurz ~ Ende",0xff
	.db	0xfe,11,"Modus: ",0xff
	.db	0xfe,12,"Sende-Intervall:",0xff,0xff
	.db	0xfe,13,"Alarm-Zeit:",0xff
	.db	0xfe,14,"Sprache/Language:",0xff
	.db	0xfe,15,"Deutsch",0xff
	.db	0xfe,16," ms",0xff		;Millisekunden
;
	.db	0xfe,0xfe			;Ende des Textbereiches
;
; English texts
;
entext:	.db	0xfe,0,"Sensor module2 v1.00",0xff,0xff
	.db	0xfe,1,"Sensors: ",0xff
	.db	0xfe,2,"Map    : ",0xff
	.db	0xfe,3,"Mode   : ",0xff
	.db	0xfe,4,"Direct",0xff,0xff
	.db	0xfe,5,"Memory",0xff,0xff
	.db	0xfe,6,1," ",0xff,0xff		;Arrow right
	.db	0xfe,7," ",0,0xff,0xff		;Arrow left
	.db	0xfe,8,"Long  ~ change",0xff,0xff
	.db	0xfe,9,"Short ~ next",0xff,0xff
	.db	0xfe,10,"Short ~ end",0xff
	.db	0xfe,11,"Mode: ",0xff,0xff
	.db	0xfe,12,"Transmit interval:",0xff,0xff
	.db	0xfe,13,"Alarm time:",0xff
	.db	0xfe,14,"Language/Sprache:",0xff
	.db	0xfe,15,"English",0xff
	.db	0xfe,16," ms",0xff		;Milliseconds
;
	.db	0xfe,0xfe			;End of text area
;
; Sensortexte: Diese knnen gendert werden. Jeder Name muss 4 Zeichen
; lang sein. Umlaute mssen entsprechend des LCD-Zeichensatzes kodiert
; werden (im Zweifelsfall bitte im Datenblatt nachschauen):
; = 0xe1, = 0xef, = 0xf5, = 0xe2
;
; Sensor texts: These can be modified. Each name must have a length of
; 4 characters:
;
sentxt:	.db	"Sen1","Sen2","Sen3","Sen4"	;Sensornamen 1-4
	.db	"Sen5","Sen6","Sen7","Sen8"	;Sensornamen 5-8
	.db	"SenA","SenB","SenC","SenD"	;Sensornamen a-d
	.db	"SenE","SenF","SenG","SenH"	;Sensornamen e-h
;
; Alarmtexte: Diese knnen gendert werden. Jeder Name muss 12 Zeichen
; lang sein. Umlaute mssen entsprechend des LCD-Zeichensatzes kodiert
; werden (im Zweifelsfall bitte im Datenblatt nachschauen):
; = 0xe1, = 0xef, = 0xf5, = 0xe2
;
; Alarm texts: These can be modified. Each name must have a length of
; 12 characters:
;
almtxt:	.db	"Alarm 1     "			;Alarmname 1
	.db	"Alarm 2     "			;Alarmname 2
	.db	"Alarm 3     "			;Alarmname 3
	.db	"Alarm 4     "			;Alarmname 4
;
;
; ------------
; EEPROM-Daten
; ------------
;
.eseg
;
dummy:	.byte	1	;Byte 0 wird nicht genutzt
emodfl:	.byte	1	;Modus-Flag, siehe Variable "modflg"
eintva:	.byte	1	;Sende-Intervall, siehe Variable "intval"
ealtim:	.byte	1	;Alarm-Zeit, siehe Variable "altime"
elang:	.byte	1	;Men-Sprache, siehe Variable "lang"
esenid:	.byte	16*8	;8-Byte-ROM-IDs, siehe Variable "sensid"
esentb:	.byte	16	;Sensor-Tabelle, siehe Variable "senstb"
